@@ -1,21 +1,13 @@
-/* $Id: network.c 4121 2006-03-26 19:32:00Z tron $ */
+/* $Id: network.c 11114 2007-09-15 11:31:37Z rubidium $ */
 
 #include "stdafx.h"
-#include "openttd.h"
-#include "debug.h"
-#include "functions.h"
-#include "string.h"
-#include "strings.h"
-#include "map.h"
 #include "network_data.h"
-#include "command.h"
-#include "variables.h"
 
 #if defined(WITH_REV)
 	extern const char _openttd_revision[];
 #elif defined(WITH_REV_HACK)
 	#define WITH_REV
-	const char _openttd_revision[] = "0.4.7";
+	const char _openttd_revision[] = "0.5.3";
 #else
 	const char _openttd_revision[] = NOREV_STRING;
 #endif
@@ -23,6 +15,15 @@
 
 #ifdef ENABLE_NETWORK
 
+#include "openttd.h"
+#include "debug.h"
+#include "functions.h"
+#include "string.h"
+#include "strings.h"
+#include "map.h"
+#include "command.h"
+#include "variables.h"
+#include "date.h"
 #include "table/strings.h"
 #include "network_client.h"
 #include "network_server.h"
@@ -40,10 +41,6 @@ struct Library *SocketBase = NULL;
 // The listen socket for the server
 static SOCKET _listensocket;
 
-// Network copy of patches, so the patches of a client are not fucked up
-//  after he joined a server
-static Patches network_tmp_patches;
-
 // The amount of clients connected
 static byte _network_clients_connected = 0;
 // The index counter for new clients (is never decreased)
@@ -58,9 +55,9 @@ NetworkClientInfo *NetworkFindClientInfoFromIndex(uint16 client_index)
 {
 	NetworkClientInfo *ci;
 
-	for (ci = _network_client_info; ci != &_network_client_info[MAX_CLIENT_INFO]; ci++)
-		if (ci->client_index == client_index)
-			return ci;
+	for (ci = _network_client_info; ci != endof(_network_client_info); ci++) {
+		if (ci->client_index == client_index) return ci;
+	}
 
 	return NULL;
 }
@@ -73,7 +70,7 @@ NetworkClientInfo *NetworkFindClientInfoFromIP(const char *ip)
 	NetworkClientInfo *ci;
 	uint32 ip_number = inet_addr(ip);
 
-	for (ci = _network_client_info; ci != &_network_client_info[MAX_CLIENT_INFO]; ci++) {
+	for (ci = _network_client_info; ci != endof(_network_client_info); ci++) {
 		if (ci->client_ip == ip_number) return ci;
 	}
 
@@ -85,9 +82,9 @@ NetworkClientState *NetworkFindClientStateFromIndex(uint16 client_index)
 {
 	NetworkClientState *cs;
 
-	for (cs = _clients; cs != &_clients[MAX_CLIENT_INFO]; cs++)
-		if (cs->index == client_index)
-			return cs;
+	for (cs = _clients; cs != endof(_clients); cs++) {
+		if (cs->index == client_index) return cs;
+	}
 
 	return NULL;
 }
@@ -96,11 +93,13 @@ NetworkClientState *NetworkFindClientStateFromIndex(uint16 client_index)
 //  if the user did not send it yet, Client #<no> is used.
 void NetworkGetClientName(char *client_name, size_t size, const NetworkClientState *cs)
 {
-	NetworkClientInfo *ci = DEREF_CLIENT_INFO(cs);
-	if (ci->client_name[0] == '\0')
-		snprintf(client_name, size, "Client #%d", cs->index);
-	else
-		snprintf(client_name, size, "%s", ci->client_name);
+	const NetworkClientInfo *ci = DEREF_CLIENT_INFO(cs);
+
+	if (ci->client_name[0] == '\0') {
+		snprintf(client_name, size, "Client #%4d", cs->index);
+	} else {
+		ttd_strlcpy(client_name, ci->client_name, size);
+	}
 }
 
 byte NetworkSpectatorCount(void)
@@ -109,7 +108,7 @@ byte NetworkSpectatorCount(void)
 	byte count = 0;
 
 	FOR_ALL_CLIENTS(cs) {
-		if (DEREF_CLIENT_INFO(cs)->client_playas == OWNER_SPECTATOR) count++;
+		if (DEREF_CLIENT_INFO(cs)->client_playas == PLAYER_SPECTATOR) count++;
 	}
 
 	return count;
@@ -131,56 +130,53 @@ void CDECL NetworkTextMessage(NetworkAction action, uint16 color, bool self_send
 	va_end(va);
 
 	switch (action) {
+		case NETWORK_ACTION_SERVER_MESSAGE:
+			color = 1;
+			snprintf(message, sizeof(message), "*** %s", buf);
+			break;
 		case NETWORK_ACTION_JOIN:
-			GetString(temp, STR_NETWORK_CLIENT_JOINED);
+			color = 1;
+			GetString(temp, STR_NETWORK_CLIENT_JOINED, lastof(temp));
 			snprintf(message, sizeof(message), "*** %s %s", name, temp);
 			break;
 		case NETWORK_ACTION_LEAVE:
-			GetString(temp, STR_NETWORK_ERR_LEFT);
+			color = 1;
+			GetString(temp, STR_NETWORK_ERR_LEFT, lastof(temp));
 			snprintf(message, sizeof(message), "*** %s %s (%s)", name, temp, buf);
 			break;
 		case NETWORK_ACTION_GIVE_MONEY:
 			if (self_send) {
 				SetDParamStr(0, name);
 				SetDParam(1, atoi(buf));
-				GetString(temp, STR_NETWORK_GAVE_MONEY_AWAY);
+				GetString(temp, STR_NETWORK_GAVE_MONEY_AWAY, lastof(temp));
 				snprintf(message, sizeof(message), "*** %s", temp);
 			} else {
 				SetDParam(0, atoi(buf));
-				GetString(temp, STR_NETWORK_GIVE_MONEY);
+				GetString(temp, STR_NETWORK_GIVE_MONEY, lastof(temp));
 				snprintf(message, sizeof(message), "*** %s %s", name, temp);
 			}
 			break;
-		case NETWORK_ACTION_CHAT_PLAYER:
-			if (self_send) {
-				SetDParamStr(0, name);
-				GetString(temp, STR_NETWORK_CHAT_TO_COMPANY);
-				snprintf(message, sizeof(message), "%s %s", temp, buf);
-			} else {
-				SetDParamStr(0, name);
-				GetString(temp, STR_NETWORK_CHAT_COMPANY);
-				snprintf(message, sizeof(message), "%s %s", temp, buf);
-			}
+		case NETWORK_ACTION_NAME_CHANGE:
+			GetString(temp, STR_NETWORK_NAME_CHANGE, lastof(temp));
+			snprintf(message, sizeof(message), "*** %s %s %s", name, temp, buf);
+			break;
+		case NETWORK_ACTION_CHAT_COMPANY:
+			SetDParamStr(0, name);
+			SetDParamStr(1, buf);
+			GetString(temp, self_send ? STR_NETWORK_CHAT_TO_COMPANY : STR_NETWORK_CHAT_COMPANY, lastof(temp));
+			ttd_strlcpy(message, temp, sizeof(message));
 			break;
 		case NETWORK_ACTION_CHAT_CLIENT:
-			if (self_send) {
-				SetDParamStr(0, name);
-				GetString(temp, STR_NETWORK_CHAT_TO_CLIENT);
-				snprintf(message, sizeof(message), "%s %s", temp, buf);
-			} else {
-				SetDParamStr(0, name);
-				GetString(temp, STR_NETWORK_CHAT_CLIENT);
-				snprintf(message, sizeof(message), "%s %s", temp, buf);
-			}
-			break;
-		case NETWORK_ACTION_NAME_CHANGE:
-			GetString(temp, STR_NETWORK_NAME_CHANGE);
-			snprintf(message, sizeof(message), "*** %s %s %s", name, temp, buf);
+			SetDParamStr(0, name);
+			SetDParamStr(1, buf);
+			GetString(temp, self_send ? STR_NETWORK_CHAT_TO_CLIENT : STR_NETWORK_CHAT_CLIENT, lastof(temp));
+			ttd_strlcpy(message, temp, sizeof(message));
 			break;
 		default:
 			SetDParamStr(0, name);
-			GetString(temp, STR_NETWORK_CHAT_ALL);
-			snprintf(message, sizeof(message), "%s %s", temp, buf);
+			SetDParamStr(1, buf);
+			GetString(temp, STR_NETWORK_CHAT_ALL, lastof(temp));
+			ttd_strlcpy(message, temp, sizeof(message));
 			break;
 	}
 
@@ -222,14 +218,15 @@ static void ServerStartError(const char *error)
 	NetworkError(STR_NETWORK_ERR_SERVER_START);
 }
 
-static void NetworkClientError(byte res, NetworkClientState *cs) {
+static void NetworkClientError(NetworkRecvStatus res, NetworkClientState* cs)
+{
 	// First, send a CLIENT_ERROR to the server, so he knows we are
 	//  disconnection (and why!)
 	NetworkErrorCode errorno;
 
 	// We just want to close the connection..
 	if (res == NETWORK_RECV_STATUS_CLOSE_QUERY) {
-		cs->quited = true;
+		cs->has_quit = true;
 		NetworkCloseClient(cs);
 		_networking = false;
 
@@ -237,10 +234,11 @@ static void NetworkClientError(byte res, NetworkClientState *cs) {
 		return;
 	}
 
-	switch(res) {
-		case NETWORK_RECV_STATUS_DESYNC: errorno = NETWORK_ERROR_DESYNC; break;
-		case NETWORK_RECV_STATUS_SAVEGAME: errorno = NETWORK_ERROR_SAVEGAME_FAILED; break;
-		default: errorno = NETWORK_ERROR_GENERAL;
+	switch (res) {
+		case NETWORK_RECV_STATUS_DESYNC:          errorno = NETWORK_ERROR_DESYNC; break;
+		case NETWORK_RECV_STATUS_SAVEGAME:        errorno = NETWORK_ERROR_SAVEGAME_FAILED; break;
+		case NETWORK_RECV_STATUS_NEWGRF_MISMATCH: errorno = NETWORK_ERROR_NEWGRF_MISMATCH; break;
+		default:                                  errorno = NETWORK_ERROR_GENERAL; break;
 	}
 	// This means we fucked up and the server closed the connection
 	if (res != NETWORK_RECV_STATUS_SERVER_ERROR && res != NETWORK_RECV_STATUS_SERVER_FULL &&
@@ -256,10 +254,77 @@ static void NetworkClientError(byte res, NetworkClientState *cs) {
 	_networking = false;
 }
 
+/** Retrieve a string representation of an internal error number
+ * @param buf buffer where the error message will be stored
+ * @param err NetworkErrorCode
+ * @return returns a pointer to the error message (buf) */
+char* GetNetworkErrorMsg(char* buf, NetworkErrorCode err, const char* last)
+{
+	/* List of possible network errors, used by
+	 * PACKET_SERVER_ERROR and PACKET_CLIENT_ERROR */
+	static const StringID network_error_strings[] = {
+		STR_NETWORK_ERR_CLIENT_GENERAL,
+		STR_NETWORK_ERR_CLIENT_DESYNC,
+		STR_NETWORK_ERR_CLIENT_SAVEGAME,
+		STR_NETWORK_ERR_CLIENT_CONNECTION_LOST,
+		STR_NETWORK_ERR_CLIENT_PROTOCOL_ERROR,
+		STR_NETWORK_ERR_CLIENT_NEWGRF_MISMATCH,
+		STR_NETWORK_ERR_CLIENT_NOT_AUTHORIZED,
+		STR_NETWORK_ERR_CLIENT_NOT_EXPECTED,
+		STR_NETWORK_ERR_CLIENT_WRONG_REVISION,
+		STR_NETWORK_ERR_CLIENT_NAME_IN_USE,
+		STR_NETWORK_ERR_CLIENT_WRONG_PASSWORD,
+		STR_NETWORK_ERR_CLIENT_PLAYER_MISMATCH,
+		STR_NETWORK_ERR_CLIENT_KICKED,
+		STR_NETWORK_ERR_CLIENT_CHEATER,
+		STR_NETWORK_ERR_CLIENT_SERVER_FULL
+	};
+
+	if (err >= lengthof(network_error_strings)) err = 0;
+
+	return GetString(buf, network_error_strings[err], last);
+}
+
+/* Count the number of active clients connected */
+static uint NetworkCountPlayers(void)
+{
+	const NetworkClientState *cs;
+	uint count = 0;
+
+	FOR_ALL_CLIENTS(cs) {
+		const NetworkClientInfo *ci = DEREF_CLIENT_INFO(cs);
+		if (IsValidPlayer(ci->client_playas)) count++;
+	}
+
+	return count;
+}
+
+static bool _min_players_paused = false;
+
+/* Check if the minimum number of players has been reached and pause or unpause the game as appropriate */
+void CheckMinPlayers(void)
+{
+	if (!_network_dedicated) return;
+
+	if (NetworkCountPlayers() < _network_min_players) {
+		if (_min_players_paused) return;
+
+		_min_players_paused = true;
+		DoCommandP(0, 1, 0, NULL, CMD_PAUSE);
+		NetworkServer_HandleChat(NETWORK_ACTION_SERVER_MESSAGE, DESTTYPE_BROADCAST, 0, "Game paused (not enough players)", NETWORK_SERVER_INDEX);
+	} else {
+		if (!_min_players_paused) return;
+
+		_min_players_paused = false;
+		DoCommandP(0, 0, 0, NULL, CMD_PAUSE);
+		NetworkServer_HandleChat(NETWORK_ACTION_SERVER_MESSAGE, DESTTYPE_BROADCAST, 0, "Game unpaused (enough players)", NETWORK_SERVER_INDEX);
+	}
+}
+
 // Find all IP-aliases for this host
 static void NetworkFindIPs(void)
 {
-	int i, last;
+	int i;
 
 #if defined(BEOS_NET_SERVER) /* doesn't have neither getifaddrs or net/if.h */
 	/* Based on Andrew Bachmann's netstat+.c. Big thanks to him! */
@@ -281,7 +346,7 @@ static void NetworkFindIPs(void)
 	i = 0;
 
 	// If something fails, make sure the list is empty
-	_network_ip_list[0] = 0;
+	_broadcast_list[0] = 0;
 
 	if (sock < 0) {
 		DEBUG(net, 0)("Error creating socket!");
@@ -300,15 +365,22 @@ static void NetworkFindIPs(void)
 			uint32 n, fields, read;
 			uint8 i1, i2, i3, i4, j1, j2, j3, j4;
 			struct in_addr inaddr;
+			uint32 ip;
+			uint32 netmask;
+
 			fields = sscanf(*output, "%u: %hhu.%hhu.%hhu.%hhu, netmask %hhu.%hhu.%hhu.%hhu%n",
 												&n, &i1,&i2,&i3,&i4, &j1,&j2,&j3,&j4, &read);
 			read += 1;
 			if (fields != 9) {
 				break;
 			}
-			inaddr.s_addr = htonl((uint32)i1 << 24 | (uint32)i2 << 16 | (uint32)i3 << 8 | (uint32)i4);
-			if (inaddr.s_addr != 0) {
-				_network_ip_list[i] = inaddr.s_addr;
+
+			ip      = (uint32)i1 << 24 | (uint32)i2 << 16 | (uint32)i3 << 8 | (uint32)i4;
+			netmask = (uint32)j1 << 24 | (uint32)j2 << 16 | (uint32)j3 << 8 | (uint32)j4;
+
+			if (ip != INADDR_LOOPBACK && ip != INADDR_ANY) {
+				inaddr.s_addr = htonl(ip | ~netmask);
+				_broadcast_list[i] = inaddr.s_addr;
 				i++;
 			}
 			if (read < 0) {
@@ -325,94 +397,99 @@ static void NetworkFindIPs(void)
 	struct ifaddrs *ifap, *ifa;
 
 	// If something fails, make sure the list is empty
-	_network_ip_list[0] = 0;
+	_broadcast_list[0] = 0;
 
 	if (getifaddrs(&ifap) != 0)
 		return;
 
 	i = 0;
 	for (ifa = ifap; ifa != NULL; ifa = ifa->ifa_next) {
-		if (ifa->ifa_addr == NULL || ifa->ifa_addr->sa_family != AF_INET)
-			continue;
-		_network_ip_list[i] = ((struct sockaddr_in *)ifa->ifa_addr)->sin_addr.s_addr;
+		if (!(ifa->ifa_flags & IFF_BROADCAST)) continue;
+		if (ifa->ifa_broadaddr == NULL) continue;
+		if (ifa->ifa_broadaddr->sa_family != AF_INET) continue;
+		_broadcast_list[i] = ((struct sockaddr_in*)ifa->ifa_broadaddr)->sin_addr.s_addr;
 		i++;
 	}
 	freeifaddrs(ifap);
 
 #else /* not HAVE_GETIFADDRS */
-
-	unsigned long len = 0;
 	SOCKET sock;
-	IFREQ ifo[MAX_INTERFACES];
-
-#ifndef WIN32
-	struct ifconf if_conf;
+#ifdef WIN32
+	DWORD len = 0;
+	INTERFACE_INFO ifo[MAX_INTERFACES];
+	uint j;
+#else
+	char buf[4 * 1024]; // Arbitrary buffer size
+	struct ifconf ifconf;
+	const char* buf_end;
+	const char* p;
 #endif
 
 	// If something fails, make sure the list is empty
-	_network_ip_list[0] = 0;
+	_broadcast_list[0] = 0;
 
 	sock = socket(AF_INET, SOCK_DGRAM, 0);
 	if (sock == INVALID_SOCKET) return;
 
 #ifdef WIN32
-	// On windows it is easy
 	memset(&ifo[0], 0, sizeof(ifo));
 	if ((WSAIoctl(sock, SIO_GET_INTERFACE_LIST, NULL, 0, &ifo[0], sizeof(ifo), &len, NULL, NULL)) != 0) {
 		closesocket(sock);
 		return;
 	}
+
+	i = 0;
+	for (j = 0; j < len / sizeof(*ifo); j++) {
+		if (ifo[j].iiFlags & IFF_LOOPBACK) continue;
+		if (!(ifo[j].iiFlags & IFF_BROADCAST)) continue;
+		/* iiBroadcast is unusable, because it always seems to be set to
+		 * 255.255.255.255.
+		 */
+		_broadcast_list[i++] =
+			 ifo[j].iiAddress.AddressIn.sin_addr.s_addr |
+			~ifo[j].iiNetmask.AddressIn.sin_addr.s_addr;
+	}
 #else
-	// On linux a bit harder
-	if_conf.ifc_len = (sizeof (struct ifreq)) * MAX_INTERFACES;
-	if_conf.ifc_buf = (char *)&ifo[0];
-	if ((ioctl(sock, SIOCGIFCONF, &if_conf)) == -1) {
+	ifconf.ifc_len = sizeof(buf);
+	ifconf.ifc_buf = buf;
+	if (ioctl(sock, SIOCGIFCONF, &ifconf) == -1) {
 		closesocket(sock);
 		return;
 	}
-	len = if_conf.ifc_len;
-#endif /* WIN32 */
 
-	// Now walk through all IPs and list them
-	for (i = 0; i < (int)(len / sizeof(IFREQ)); i++) {
-		// Request IP for this interface
-#ifdef WIN32
-		_network_ip_list[i] = *(&ifo[i].iiAddress.AddressIn.sin_addr.s_addr);
-#else
-		if ((ioctl(sock, SIOCGIFADDR, &ifo[i])) != 0) {
-			closesocket(sock);
-			return;
+	i = 0;
+	buf_end = buf + ifconf.ifc_len;
+	for (p = buf; p < buf_end;) {
+		const struct ifreq* req = (const struct ifreq*)p;
+
+		if (req->ifr_addr.sa_family == AF_INET) {
+			struct ifreq r;
+
+			strncpy(r.ifr_name, req->ifr_name, lengthof(r.ifr_name));
+			if (ioctl(sock, SIOCGIFFLAGS, &r) != -1 &&
+					r.ifr_flags & IFF_BROADCAST &&
+					ioctl(sock, SIOCGIFBRDADDR, &r) != -1) {
+				_broadcast_list[i++] =
+					((struct sockaddr_in*)&r.ifr_broadaddr)->sin_addr.s_addr;
+			}
 		}
 
-		_network_ip_list[i] = ((struct sockaddr_in *)&ifo[i].ifr_addr)->sin_addr.s_addr;
+		p += sizeof(struct ifreq);
+#ifdef AF_LINK
+		p += req->ifr_addr.sa_len - sizeof(struct sockaddr);
 #endif
 	}
+#endif
 
 	closesocket(sock);
-
 #endif /* not HAVE_GETIFADDRS */
 
-	_network_ip_list[i] = 0;
-	last = i - 1;
+	_broadcast_list[i] = 0;
 
-	DEBUG(net, 3)("Detected IPs:");
+	DEBUG(net, 3)("Detected broadcast addresses:");
 	// Now display to the debug all the detected ips
-	i = 0;
-	while (_network_ip_list[i] != 0) {
-		// Also check for non-used ips (127.0.0.1)
-		if (_network_ip_list[i] == inet_addr("127.0.0.1")) {
-			// If there is an ip after thisone, put him in here
-			if (last > i)
-				_network_ip_list[i] = _network_ip_list[last];
-			// Clear the last ip
-			_network_ip_list[last] = 0;
-			// And we have 1 ip less
-			last--;
-			continue;
-		}
-
-		DEBUG(net, 3)(" %d) %s", i, inet_ntoa(*(struct in_addr *)&_network_ip_list[i]));//inet_ntoa(inaddr));
-		i++;
+	for (i = 0; _broadcast_list[i] != 0; i++) {
+		DEBUG(net, 3)(" %d) %s", i, inet_ntoa(*(struct in_addr *)&_broadcast_list[i]));//inet_ntoa(inaddr));
 	}
 }
 
@@ -438,22 +515,25 @@ unsigned long NetworkResolveHost(const char *hostname)
 	return ip;
 }
 
-// Converts a string to ip/port/player
-//  Format: IP#player:port
-//
-// connection_string will be re-terminated to seperate out the hostname, and player and port will
-// be set to the player and port strings given by the user, inside the memory area originally
-// occupied by connection_string.
+/** Converts a string to ip/port/player
+ *  Format: IP#player:port
+ *
+ * connection_string will be re-terminated to seperate out the hostname, and player and port will
+ * be set to the player and port strings given by the user, inside the memory area originally
+ * occupied by connection_string. */
 void ParseConnectionString(const char **player, const char **port, char *connection_string)
 {
 	char *p;
 	for (p = connection_string; *p != '\0'; p++) {
-		if (*p == '#') {
-			*player = p + 1;
-			*p = '\0';
-		} else if (*p == ':') {
-			*port = p + 1;
-			*p = '\0';
+		switch (*p) {
+			case '#':
+				*player = p + 1;
+				*p = '\0';
+				break;
+			case ':':
+				*port = p + 1;
+				*p = '\0';
+				break;
 		}
 	}
 }
@@ -463,38 +543,33 @@ void ParseConnectionString(const char **player, const char **port, char *connect
 static NetworkClientState *NetworkAllocClient(SOCKET s)
 {
 	NetworkClientState *cs;
-	NetworkClientInfo *ci;
-	byte client_no;
-
-	client_no = 0;
+	byte client_no = 0;
 
 	if (_network_server) {
 		// Can we handle a new client?
-		if (_network_clients_connected >= MAX_CLIENTS)
-			return NULL;
-
-		if (_network_game_info.clients_on >= _network_game_info.clients_max)
-			return NULL;
+		if (_network_clients_connected >= MAX_CLIENTS) return NULL;
+		if (_network_game_info.clients_on >= _network_game_info.clients_max) return NULL;
 
 		// Register the login
 		client_no = _network_clients_connected++;
 	}
 
-	cs = &_clients[client_no];
+	cs = DEREF_CLIENT(client_no);
 	memset(cs, 0, sizeof(*cs));
 	cs->socket = s;
 	cs->last_frame = 0;
-	cs->quited = false;
+	cs->has_quit = false;
 
 	cs->last_frame = _frame_counter;
 	cs->last_frame_server = _frame_counter;
 
 	if (_network_server) {
-		ci = DEREF_CLIENT_INFO(cs);
+		NetworkClientInfo *ci = DEREF_CLIENT_INFO(cs);
 		memset(ci, 0, sizeof(*ci));
 
 		cs->index = _network_client_index++;
 		ci->client_index = cs->index;
+		ci->client_playas = PLAYER_INACTIVE_CLIENT;
 		ci->join_date = _date;
 
 		InvalidateWindow(WC_CLIENT_LIST, 0);
@@ -509,22 +584,22 @@ void NetworkCloseClient(NetworkClientState *cs)
 	NetworkClientInfo *ci;
 	// Socket is already dead
 	if (cs->socket == INVALID_SOCKET) {
-		cs->quited = true;
+		cs->has_quit = true;
 		return;
 	}
 
 	DEBUG(net, 1) ("[NET] Closed client connection");
 
-	if (!cs->quited && _network_server && cs->status > STATUS_INACTIVE) {
+	if (!cs->has_quit && _network_server && cs->status > STATUS_INACTIVE) {
 		// We did not receive a leave message from this client...
 		NetworkErrorCode errorno = NETWORK_ERROR_CONNECTION_LOST;
 		char str[100];
-		char client_name[NETWORK_NAME_LENGTH];
+		char client_name[NETWORK_CLIENT_NAME_LENGTH];
 		NetworkClientState *new_cs;
 
 		NetworkGetClientName(client_name, sizeof(client_name), cs);
 
-		GetString(str, STR_NETWORK_ERR_CLIENT_GENERAL + errorno);
+		GetNetworkErrorMsg(str, errorno, lastof(str));
 
 		NetworkTextMessage(NETWORK_ACTION_LEAVE, 1, false, client_name, "%s", str);
 
@@ -539,12 +614,12 @@ void NetworkCloseClient(NetworkClientState *cs)
 	/* When the client was PRE_ACTIVE, the server was in pause mode, so unpause */
 	if (cs->status == STATUS_PRE_ACTIVE && _network_pause_on_join) {
 		DoCommandP(0, 0, 0, NULL, CMD_PAUSE);
-		NetworkServer_HandleChat(NETWORK_ACTION_CHAT, DESTTYPE_BROADCAST, 0, "Game unpaused", NETWORK_SERVER_INDEX);
+		NetworkServer_HandleChat(NETWORK_ACTION_SERVER_MESSAGE, DESTTYPE_BROADCAST, 0, "Game unpaused", NETWORK_SERVER_INDEX);
 	}
 
 	closesocket(cs->socket);
 	cs->writable = false;
-	cs->quited = true;
+	cs->has_quit = true;
 
 	// Free all pending and partially received packets
 	while (cs->packet_queue != NULL) {
@@ -566,7 +641,7 @@ void NetworkCloseClient(NetworkClientState *cs)
 
 	if (_network_server) {
 		// We just lost one client :(
-		if (cs->status > STATUS_INACTIVE) _network_game_info.clients_on--;
+		if (cs->status >= STATUS_AUTH) _network_game_info.clients_on--;
 		_network_clients_connected--;
 
 		while ((cs + 1) != DEREF_CLIENT(MAX_CLIENTS) && (cs + 1)->socket != INVALID_SOCKET) {
@@ -584,6 +659,8 @@ void NetworkCloseClient(NetworkClientState *cs)
 	cs->status = STATUS_INACTIVE;
 	cs->index = NETWORK_EMPTY_INDEX;
 	ci->client_index = NETWORK_EMPTY_INDEX;
+
+	CheckMinPlayers();
 }
 
 // A client wants to connect to a server
@@ -600,29 +677,23 @@ static bool NetworkConnect(const char *hostname, int port)
 		return false;
 	}
 
-	if (!SetNoDelay(s))
-		DEBUG(net, 1)("[NET] Setting TCP_NODELAY failed");
+	if (!SetNoDelay(s)) DEBUG(net, 1)("[NET] Setting TCP_NODELAY failed");
 
 	sin.sin_family = AF_INET;
 	sin.sin_addr.s_addr = NetworkResolveHost(hostname);
 	sin.sin_port = htons(port);
 	_network_last_host_ip = sin.sin_addr.s_addr;
 
-	if (connect(s, (struct sockaddr*) &sin, sizeof(sin)) != 0) {
-		// We failed to connect for which reason what so ever
-		return false;
-	}
+	/* We failed to connect for which reason what so ever */
+	if (connect(s, (struct sockaddr*) &sin, sizeof(sin)) != 0) return false;
 
-	if (!SetNonBlocking(s))
-		DEBUG(net, 0)("[NET] Setting non-blocking failed"); // XXX should this be an error?
+	if (!SetNonBlocking(s)) DEBUG(net, 0)("[NET] Setting non-blocking failed"); // XXX should this be an error?
 
 	// in client mode, only the first client field is used. it's pointing to the server.
 	NetworkAllocClient(s);
 
 	ShowJoinStatusWindow();
 
-	memcpy(&network_tmp_patches, &_patches, sizeof(_patches));
-
 	return true;
 }
 
@@ -630,7 +701,6 @@ static bool NetworkConnect(const char *hostname, int port)
 static void NetworkAcceptClients(void)
 {
 	struct sockaddr_in sin;
-	SOCKET s;
 	NetworkClientState *cs;
 	uint i;
 	bool banned;
@@ -639,10 +709,8 @@ static void NetworkAcceptClients(void)
 	assert(_listensocket != INVALID_SOCKET);
 
 	for (;;) {
-		socklen_t sin_len;
-
-		sin_len = sizeof(sin);
-		s = accept(_listensocket, (struct sockaddr*)&sin, &sin_len);
+		socklen_t sin_len = sizeof(sin);
+		SOCKET s = accept(_listensocket, (struct sockaddr*)&sin, &sin_len);
 		if (s == INVALID_SOCKET) return;
 
 		SetNonBlocking(s); // XXX error handling?
@@ -654,8 +722,7 @@ static void NetworkAcceptClients(void)
 		/* Check if the client is banned */
 		banned = false;
 		for (i = 0; i < lengthof(_network_ban_list); i++) {
-			if (_network_ban_list[i] == NULL)
-				continue;
+			if (_network_ban_list[i] == NULL) continue;
 
 			if (sin.sin_addr.s_addr == inet_addr(_network_ban_list[i])) {
 				Packet *p = NetworkSend_Init(PACKET_SERVER_BANNED);
@@ -675,8 +742,7 @@ static void NetworkAcceptClients(void)
 			}
 		}
 		/* If this client is banned, continue with next client */
-		if (banned)
-			continue;
+		if (banned) continue;
 
 		cs = NetworkAllocClient(s);
 		if (cs == NULL) {
@@ -700,12 +766,7 @@ static void NetworkAcceptClients(void)
 		//  the client stays inactive
 		cs->status = STATUS_INACTIVE;
 
-		{
-			// Save the IP of the client
-			NetworkClientInfo *ci;
-			ci = DEREF_CLIENT_INFO(cs);
-			ci->client_ip = sin.sin_addr.s_addr;
-		}
+		DEREF_CLIENT_INFO(cs)->client_ip = sin.sin_addr.s_addr; // Save the IP of the client
 	}
 }
 
@@ -714,11 +775,8 @@ static bool NetworkListen(void)
 {
 	SOCKET ls;
 	struct sockaddr_in sin;
-	int port;
 
-	port = _network_server_port;
-
-	DEBUG(net, 1) ("[NET] Listening on %s:%d", _network_server_bind_ip_host, port);
+	DEBUG(net, 1) ("[NET] Listening on %s:%d", _network_server_bind_ip_host, _network_server_port);
 
 	ls = socket(AF_INET, SOCK_STREAM, 0);
 	if (ls == INVALID_SOCKET) {
@@ -735,12 +793,11 @@ static bool NetworkListen(void)
 		}
 	}
 
-	if (!SetNonBlocking(ls))
-		DEBUG(net, 0)("[NET] Setting non-blocking failed"); // XXX should this be an error?
+	if (!SetNonBlocking(ls)) DEBUG(net, 0)("[NET] Setting non-blocking failed"); // XXX should this be an error?
 
 	sin.sin_family = AF_INET;
 	sin.sin_addr.s_addr = _network_server_bind_ip;
-	sin.sin_port = htons(port);
+	sin.sin_port = htons(_network_server_port);
 
 	if (bind(ls, (struct sockaddr*)&sin, sizeof(sin)) != 0) {
 		ServerStartError("bind() failed");
@@ -815,9 +872,7 @@ NetworkGameList *NetworkQueryServer(const char* host, unsigned short port, bool
 
 	NetworkDisconnect();
 
-	if (game_info) {
-		return NetworkUDPQueryServer(host, port);
-	}
+	if (game_info) return NetworkUDPQueryServer(host, port);
 
 	NetworkInitialize();
 
@@ -826,17 +881,13 @@ NetworkGameList *NetworkQueryServer(const char* host, unsigned short port, bool
 	// Try to connect
 	_networking = NetworkConnect(host, port);
 
-//	ttd_strlcpy(_network_last_host, host, sizeof(_network_last_host));
-//	_network_last_port = port;
-
 	// We are connected
 	if (_networking) {
 		SEND_COMMAND(PACKET_CLIENT_COMPANY_INFO)();
-		return NULL;
+	} else { // No networking, close everything down again
+		NetworkDisconnect();
 	}
 
-	// No networking, close everything down again
-	NetworkDisconnect();
 	return NULL;
 }
 
@@ -858,8 +909,6 @@ void NetworkAddServer(const char *b)
 		rport = NETWORK_DEFAULT_PORT;
 
 		ParseConnectionString(&player, &port, host);
-
-		if (player != NULL) _network_playas = atoi(player);
 		if (port != NULL) rport = atoi(port);
 
 		item = NetworkQueryServer(host, rport, true);
@@ -873,7 +922,7 @@ void NetworkAddServer(const char *b)
 void NetworkRebuildHostList(void)
 {
 	uint i = 0;
-	NetworkGameList *item = _network_game_list;
+	const NetworkGameList *item = _network_game_list;
 	while (item != NULL && i != lengthof(_network_host_list)) {
 		if (item->manually) {
 			free(_network_host_list[i]);
@@ -889,7 +938,7 @@ void NetworkRebuildHostList(void)
 }
 
 // Used by clients, to connect to a server
-bool NetworkClientConnectGame(const char* host, unsigned short port)
+bool NetworkClientConnectGame(const char *host, uint16 port)
 {
 	if (!_network_available) return false;
 
@@ -943,7 +992,7 @@ static void NetworkInitGameInfo(void)
 	_network_game_info.spectators_on = 0;
 
 	_network_game_info.game_date = _date;
-	_network_game_info.start_date = ConvertIntDate(_patches.starting_date);
+	_network_game_info.start_date = ConvertYMDToDate(_patches.starting_year, 0, 1);
 	_network_game_info.map_width = MapSizeX();
 	_network_game_info.map_height = MapSizeY();
 	_network_game_info.map_set = _opt.landscape;
@@ -956,10 +1005,8 @@ static void NetworkInitGameInfo(void)
 	memset(ci, 0, sizeof(*ci));
 
 	ci->client_index = NETWORK_SERVER_INDEX;
-	if (_network_dedicated)
-		ci->client_playas = OWNER_SPECTATOR;
-	else
-		ci->client_playas = _local_player + 1;
+	ci->client_playas = _network_dedicated ? PLAYER_SPECTATOR : _local_player;
+
 	ttd_strlcpy(ci->client_name, _network_player_name, sizeof(ci->client_name));
 	ttd_strlcpy(ci->unique_id, _network_unique_id, sizeof(ci->unique_id));
 }
@@ -973,8 +1020,7 @@ bool NetworkServerStart(void)
 	if (_network_dedicated) IConsoleCmdExec("exec scripts/pre_dedicated.scr 0");
 
 	NetworkInitialize();
-	if (!NetworkListen())
-		return false;
+	if (!NetworkListen()) return false;
 
 	// Try to start UDP-server
 	_network_udp_server = true;
@@ -988,8 +1034,8 @@ bool NetworkServerStart(void)
 	_last_sync_frame = 0;
 	_network_own_client_index = NETWORK_SERVER_INDEX;
 
-	if (!_network_dedicated)
-		_network_playas = 1;
+	/* Non-dedicated server will always be player #1 */
+	if (!_network_dedicated) _network_playas = 0;
 
 	_network_clients_connected = 0;
 
@@ -1000,6 +1046,9 @@ bool NetworkServerStart(void)
 	// if the server is dedicated ... add some other script
 	if (_network_dedicated) IConsoleCmdExec("exec scripts/on_dedicated.scr 0");
 
+	_min_players_paused = false;
+	CheckMinPlayers();
+
 	/* Try to register us to the master server */
 	_network_last_advertise_frame = 0;
 	_network_need_advertise = true;
@@ -1043,8 +1092,7 @@ void NetworkDisconnect(void)
 		}
 	}
 
-	if (_network_advertise)
-		NetworkUDPRemoveAdvertise();
+	if (_network_advertise) NetworkUDPRemoveAdvertise();
 
 	DeleteWindowById(WC_NETWORK_STATUS_WINDOW, 0);
 
@@ -1057,10 +1105,6 @@ void NetworkDisconnect(void)
 		free(p);
 	}
 
-	if (_networking && !_network_server) {
-		memcpy(&_patches, &network_tmp_patches, sizeof(_patches));
-	}
-
 	_networking = false;
 	_network_server = false;
 }
@@ -1082,9 +1126,7 @@ static bool NetworkReceive(void)
 	}
 
 	// take care of listener port
-	if (_network_server) {
-		FD_SET(_listensocket, &read_fd);
-	}
+	if (_network_server) FD_SET(_listensocket, &read_fd);
 
 	tv.tv_sec = tv.tv_usec = 0; // don't block at all.
 #if !defined(__MORPHOS__) && !defined(__AMIGA__)
@@ -1095,23 +1137,24 @@ static bool NetworkReceive(void)
 	if (n == -1 && !_network_server) NetworkError(STR_NETWORK_ERR_LOSTCONNECTION);
 
 	// accept clients..
-	if (_network_server && FD_ISSET(_listensocket, &read_fd))
-		NetworkAcceptClients();
+	if (_network_server && FD_ISSET(_listensocket, &read_fd)) NetworkAcceptClients();
 
 	// read stuff from clients
 	FOR_ALL_CLIENTS(cs) {
 		cs->writable = !!FD_ISSET(cs->socket, &write_fd);
 		if (FD_ISSET(cs->socket, &read_fd)) {
-			if (_network_server)
+			if (_network_server) {
 				NetworkServer_ReadPackets(cs);
-			else {
-				byte res;
+			} else {
+				NetworkRecvStatus res;
+
 				// The client already was quiting!
-				if (cs->quited) return false;
-				if ((res = NetworkClient_ReadPackets(cs)) != NETWORK_RECV_STATUS_OKAY) {
+				if (cs->has_quit) return false;
+
+				res = NetworkClient_ReadPackets(cs);
+				if (res != NETWORK_RECV_STATUS_OKAY) {
 					// The client made an error of which we can not recover
 					//   close the client and drop back to main menu
-
 					NetworkClientError(res, cs);
 					return false;
 				}
@@ -1148,8 +1191,7 @@ static void NetworkHandleLocalQueue(void)
 
 		// The queue is always in order, which means
 		// that the first element will be executed first.
-		if (_frame_counter < cp->frame)
-			break;
+		if (_frame_counter < cp->frame) break;
 
 		if (_frame_counter > cp->frame) {
 			// If we reach here, it means for whatever reason, we've already executed
@@ -1168,7 +1210,7 @@ static void NetworkHandleLocalQueue(void)
 	// Just a safety check, to be removed in the future.
 	// Make sure that no older command appears towards the end of the queue
 	// In that case we missed executing it. This will never happen.
-	for(cp = _local_command_queue; cp; cp = cp->next) {
+	for (cp = _local_command_queue; cp; cp = cp->next) {
 		assert(_frame_counter < cp->frame);
 	}
 
@@ -1222,11 +1264,10 @@ void NetworkUDPGameLoop(void)
 		if (_udp_master_socket != INVALID_SOCKET) {
 			NetworkUDPReceive(_udp_master_socket);
 		}
-	}
-	else if (_udp_client_socket != INVALID_SOCKET) {
+	} else if (_udp_client_socket != INVALID_SOCKET) {
 		NetworkUDPReceive(_udp_client_socket);
-		if (_network_udp_broadcast > 0)
-			_network_udp_broadcast--;
+		if (_network_udp_broadcast > 0) _network_udp_broadcast--;
+		NetworkGameListRequery();
 	}
 }
 
@@ -1270,9 +1311,7 @@ void NetworkGameLoop(void)
 			}
 		} else {
 			// Else, keep on going till _frame_counter_max
-			if (_frame_counter_max > _frame_counter) {
-				NetworkDoClientLoop();
-			}
+			if (_frame_counter_max > _frame_counter) NetworkDoClientLoop();
 		}
 	}
 
@@ -1306,38 +1345,40 @@ void NetworkStartUp(void)
 {
 	DEBUG(net, 3) ("[NET][Core] Starting network...");
 
-	#if defined(__MORPHOS__) || defined(__AMIGA__)
+#if defined(__MORPHOS__) || defined(__AMIGA__)
 	/*
 	 *  IMPORTANT NOTE: SocketBase needs to be initialized before we use _any_
 	 *  network related function, else: crash.
 	 */
-	{
-		DEBUG(misc,3) ("[NET][Core] Loading bsd socket library");
-		if (!(SocketBase = OpenLibrary("bsdsocket.library", 4))) {
-			DEBUG(net, 0) ("[NET][Core] Error: couldn't open bsdsocket.library version 4. Network not available.");
-			_network_available = false;
-			return;
-		}
+	DEBUG(misc,3) ("[NET][Core] Loading bsd socket library");
+	SocketBase = OpenLibrary("bsdsocket.library", 4);
+	if (SocketBase == NULL) {
+		DEBUG(net, 0) ("[NET][Core] Error: couldn't open bsdsocket.library version 4. Network not available.");
+		_network_available = false;
+		return;
+	}
 
-		#if defined(__AMIGA__)
-		// for usleep() implementation (only required for legacy AmigaOS builds)
-		if ( (TimerPort = CreateMsgPort()) ) {
-			if ( (TimerRequest = (struct timerequest *) CreateIORequest(TimerPort, sizeof(struct timerequest))) ) {
-				if ( OpenDevice("timer.device", UNIT_MICROHZ, (struct IORequest *) TimerRequest, 0) == 0 ) {
-					if ( !(TimerBase = TimerRequest->tr_node.io_Device) ) {
-						// free ressources...
-						DEBUG(net, 0) ("[NET][Core] Error: couldn't initialize timer. Network not available.");
-						_network_available = false;
-						return;
-					}
+#if defined(__AMIGA__)
+	// for usleep() implementation (only required for legacy AmigaOS builds)
+	TimerPort = CreateMsgPort();
+	if (TimerPort != NULL) {
+		TimerRequest = (struct timerequest*)CreateIORequest(TimerPort, sizeof(struct timerequest);
+		if (TimerRequest != NULL) {
+			if (OpenDevice("timer.device", UNIT_MICROHZ, (struct IORequest*)TimerRequest, 0) == 0) {
+				TimerBase = TimerRequest->tr_node.io_Device;
+				if (TimerBase == NULL) {
+					// free ressources...
+					DEBUG(net, 0) ("[NET][Core] Error: couldn't initialize timer. Network not available.");
+					_network_available = false;
+					return;
 				}
 			}
 		}
-		#endif // __AMIGA__
 	}
-	#endif // __MORPHOS__ / __AMIGA__
+#endif // __AMIGA__
+#endif // __MORPHOS__ / __AMIGA__
 
-    // Network is available
+	// Network is available
 	_network_available = true;
 	_network_dedicated = false;
 	_network_last_advertise_frame = 0;
@@ -1352,10 +1393,18 @@ void NetworkStartUp(void)
 	/* Generate an unique id when there is none yet */
 	if (_network_unique_id[0] == '\0') NetworkGenerateUniqueId();
 
-	memset(&_network_game_info, 0, sizeof(_network_game_info));
-	_network_game_info.clients_max = 10; // XXX - hardcoded, string limiation -- TrueLight
-	_network_game_info.companies_max = MAX_PLAYERS; // 8
-	_network_game_info.spectators_max = _network_game_info.clients_max;
+	{
+		byte cl_max = _network_game_info.clients_max;
+		byte cp_max = _network_game_info.companies_max;
+		byte sp_max = _network_game_info.spectators_max;
+		byte s_lang = _network_game_info.server_lang;
+
+		memset(&_network_game_info, 0, sizeof(_network_game_info));
+		_network_game_info.clients_max = cl_max;
+		_network_game_info.companies_max = cp_max;
+		_network_game_info.spectators_max = sp_max;
+		_network_game_info.server_lang = s_lang;
+	}
 
 	// Let's load the network in windows
 	#if defined(WIN32)
@@ -1378,34 +1427,27 @@ void NetworkStartUp(void)
 // This shuts the network down
 void NetworkShutDown(void)
 {
+	NetworkDisconnect();
+	NetworkUDPClose();
+
 	DEBUG(net, 3) ("[NET][Core] Shutting down the network.");
 
 	_network_available = false;
 
-	#if defined(__MORPHOS__) || defined(__AMIGA__)
-	{
-		// free allocated ressources
-		#if defined(__AMIGA__)
-			if (TimerBase)    { CloseDevice((struct IORequest *) TimerRequest); }
-			if (TimerRequest) { DeleteIORequest(TimerRequest); }
-			if (TimerPort)    { DeleteMsgPort(TimerPort); }
-		#endif
+#if defined(__MORPHOS__) || defined(__AMIGA__)
+	// free allocated ressources
+#if defined(__AMIGA__)
+	if (TimerBase    != NULL) CloseDevice((struct IORequest*)TimerRequest); // XXX This smells wrong
+	if (TimerRequest != NULL) DeleteIORequest(TimerRequest);
+	if (TimerPort    != NULL) DeleteMsgPort(TimerPort);
+#endif
 
-		if (SocketBase) {
-			CloseLibrary(SocketBase);
-		}
-	}
-	#endif
+	if (SocketBase != NULL) CloseLibrary(SocketBase);
+#endif
 
-	#if defined(WIN32)
-	{
-		WSACleanup();
-	}
-	#endif
+#if defined(WIN32)
+	WSACleanup();
+#endif
 }
-#else
-
-void ParseConnectionString(const char **player, const char **port, char *connection_string) {}
-void NetworkUpdateClientInfo(uint16 client_index) {}
 
 #endif /* ENABLE_NETWORK */
