@@ -1,4 +1,4 @@
-/* $Id: news_gui.c 3961 2006-03-18 15:46:09Z Darkvater $ */
+/* $Id: news_gui.c 9851 2007-05-15 21:42:27Z rubidium $ */
 
 #include "stdafx.h"
 #include "openttd.h"
@@ -14,62 +14,60 @@
 #include "vehicle.h"
 #include "sound.h"
 #include "variables.h"
+#include "date.h"
+#include "string.h"
 
 /* News system
-News system is realized as a FIFO queue (in an array)
-The positions in the queue can't be rearranged, we only access
-the array elements through pointers to the elements. Once the
-array is full, the oldest entry (_oldest_news) is being overwritten
-by the newest (_latest news).
-
-oldest                   current   lastest
- |                          |         |
-[O------------F-------------C---------L           ]
-              |
-           forced
-*/
+ * News system is realized as a FIFO queue (in an array)
+ * The positions in the queue can't be rearranged, we only access
+ * the array elements through pointers to the elements. Once the
+ * array is full, the oldest entry (_oldest_news) is being overwritten
+ * by the newest (_latest news).
+ *
+ * oldest                   current   lastest
+ *  |                          |         |
+ * [O------------F-------------C---------L           ]
+ *               |
+ *            forced
+ *
+ * Of course by using an array we can have situations like
+ *
+ * [----L          O-----F---------C-----------------]
+ * This is where we have wrapped around the array and have
+ * (MAX_NEWS - O) + L news items
+ */
 
 #define MAX_NEWS 30
 
+typedef byte NewsID;
 #define INVALID_NEWS 255
 
 static NewsItem _news_items[MAX_NEWS];
-static byte _current_news = INVALID_NEWS; // points to news item that should be shown next
-static byte _oldest_news = 0;    // points to first item in fifo queue
-static byte _latest_news = INVALID_NEWS;  // points to last item in fifo queue
+static NewsID _current_news = INVALID_NEWS; // points to news item that should be shown next
+static NewsID _oldest_news = 0;    // points to first item in fifo queue
+static NewsID _latest_news = INVALID_NEWS;  // points to last item in fifo queue
 /* if the message being shown was forced by the user, its index is stored in
  * _forced_news. forced_news is INVALID_NEWS otherwise.
  * (Users can force messages through history or "last message") */
-static byte _forced_news = INVALID_NEWS;
+static NewsID _forced_news = INVALID_NEWS;
 
 static byte _total_news = 0; // total news count
 
-void DrawNewsNewTrainAvail(Window *w);
-void DrawNewsNewRoadVehAvail(Window *w);
-void DrawNewsNewShipAvail(Window *w);
-void DrawNewsNewAircraftAvail(Window *w);
+void DrawNewsNewVehicleAvail(Window *w);
 void DrawNewsBankrupcy(Window *w);
-static void MoveToNexItem(void);
+static void MoveToNextItem(void);
 
-StringID GetNewsStringNewTrainAvail(const NewsItem *ni);
-StringID GetNewsStringNewRoadVehAvail(const NewsItem *ni);
-StringID GetNewsStringNewShipAvail(const NewsItem *ni);
-StringID GetNewsStringNewAircraftAvail(const NewsItem *ni);
+StringID GetNewsStringNewVehicleAvail(const NewsItem *ni);
 StringID GetNewsStringBankrupcy(const NewsItem *ni);
 
 static DrawNewsCallbackProc * const _draw_news_callback[] = {
-	DrawNewsNewTrainAvail,    /* DNC_TRAINAVAIL */
-	DrawNewsNewRoadVehAvail,  /* DNC_ROADAVAIL */
-	DrawNewsNewShipAvail,     /* DNC_SHIPAVAIL */
-	DrawNewsNewAircraftAvail, /* DNC_AIRCRAFTAVAIL */
+	DrawNewsNewVehicleAvail,  /* DNC_VEHICLEAVAIL */
 	DrawNewsBankrupcy,        /* DNC_BANKRUPCY */
 };
 
+extern GetNewsStringCallbackProc * const _get_news_string_callback[];
 GetNewsStringCallbackProc * const _get_news_string_callback[] = {
-	GetNewsStringNewTrainAvail,    /* DNC_TRAINAVAIL */
-	GetNewsStringNewRoadVehAvail,  /* DNC_ROADAVAIL */
-	GetNewsStringNewShipAvail,     /* DNC_SHIPAVAIL */
-	GetNewsStringNewAircraftAvail, /* DNC_AIRCRAFTAVAIL */
+	GetNewsStringNewVehicleAvail,  /* DNC_VEHICLEAVAIL */
 	GetNewsStringBankrupcy,        /* DNC_BANKRUPCY */
 };
 
@@ -166,7 +164,7 @@ static void NewsWindowProc(Window *w, WindowEvent *e)
 	} break;
 
 	case WE_CLICK: {
-		switch (e->click.widget) {
+		switch (e->we.click.widget) {
 		case 1: {
 			NewsItem *ni = WP(w, news_d).ni;
 			DeleteWindow(w);
@@ -187,16 +185,16 @@ static void NewsWindowProc(Window *w, WindowEvent *e)
 	} break;
 
 	case WE_KEYPRESS:
-		if (e->keypress.keycode == WKC_SPACE) {
+		if (e->we.keypress.keycode == WKC_SPACE) {
 			// Don't continue.
-			e->keypress.cont = false;
+			e->we.keypress.cont = false;
 			DeleteWindow(w);
 		}
 		break;
 
 	case WE_MESSAGE: /* The chatbar has notified us that is was either created or closed */
-		switch (e->message.msg) {
-			case WE_CREATE: w->message.msg = e->message.wparam; break;
+		switch (e->we.message.msg) {
+			case WE_CREATE: w->message.msg = e->we.message.wparam; break;
 			case WE_DESTROY: w->message.msg = 0; break;
 		}
 		break;
@@ -217,74 +215,97 @@ static void NewsWindowProc(Window *w, WindowEvent *e)
 	}
 }
 
-// returns the correct index in the array
-// (to deal with overflows)
-static byte increaseIndex(byte i)
+/** Return the correct index in the pseudo-fifo
+ * queue and deals with overflows when increasing the index */
+static inline NewsID increaseIndex(NewsID i)
 {
-	if (i == INVALID_NEWS)
-		return 0;
-	i++;
-	if (i >= MAX_NEWS)
-		i = i % MAX_NEWS;
-	return i;
+	assert(i != INVALID_NEWS);
+	return (i + 1) % MAX_NEWS;
 }
 
+/** Return the correct index in the pseudo-fifo
+ * queue and deals with overflows when decreasing the index */
+static inline NewsID decreaseIndex(NewsID i)
+{
+	assert(i != INVALID_NEWS);
+	return (i + MAX_NEWS - 1) % MAX_NEWS;
+}
+
+/** Add a new newsitem to be shown.
+ * @param string String to display, can have special values based on parameter 'flags'
+ * @param flags various control bits that will show various news-types. See macro NEWS_FLAGS()
+ * @param data_a news-specific value based on news type
+ * @param data_b news-specific value based on news type
+ * @note flags exists of 4 byte-sized extra parameters.<br/>
+ * 1.  0 -  7 display_mode, any of the NewsMode enums (NM_)<br/>
+ * 2.  8 - 15 news flags, any of the NewsFlags enums (NF_) NF_INCOLOR are set automatically if needed<br/>
+ * 3. 16 - 23 news category, any of the NewsType enums (NT_)<br/>
+ * 4. 24 - 31 news callback function, any of the NewsCallback enums (DNC_)<br/>
+ * If the display mode is NM_CALLBACK special news is shown and parameter
+ * stringid has a special meaning.<br/>
+ * DNC_TRAINAVAIL, DNC_ROADAVAIL, DNC_SHIPAVAIL, DNC_AIRCRAFTAVAIL: StringID is
+ * the index of the engine that is shown<br/>
+ * DNC_BANKRUPCY: bytes 0-3 of StringID contains the player that is in trouble,
+ * and 4-7 contains what kind of bankrupcy message is shown, NewsBankrupcy enum (NB_)<br/>
+ * @see NewsMode
+ * @see NewsFlags
+ * @see NewsType
+ * @see NewsCallback */
 void AddNewsItem(StringID string, uint32 flags, uint data_a, uint data_b)
 {
-	NewsItem *ni;
-	Window *w;
+	NewsID l_news;
 
-	if (_game_mode == GM_MENU)
-		return;
+	if (_game_mode == GM_MENU) return;
 
 	// check the rare case that the oldest (to be overwritten) news item is open
-	if (_total_news==MAX_NEWS && (_oldest_news == _current_news || _oldest_news == _forced_news))
-		MoveToNexItem();
+	if (_total_news == MAX_NEWS && (_oldest_news == _current_news || _oldest_news == _forced_news))
+		MoveToNextItem();
 
-	_forced_news = INVALID_NEWS;
 	if (_total_news < MAX_NEWS) _total_news++;
 
-	// make sure our pointer isn't overflowing
-	_latest_news = increaseIndex(_latest_news);
+	/* Increase _latest_news. If we have no news yet, use _oldest news as an
+	 * index. We cannot use 0 as _oldest_news can jump around due to
+	 * DeleteVehicleNews */
+	l_news = _latest_news;
+	_latest_news = (_latest_news == INVALID_NEWS) ? _oldest_news : increaseIndex(_latest_news);
 
-	// overwrite oldest news entry
-	if (_oldest_news == _latest_news && _news_items[_oldest_news].string_id != 0)
-		_oldest_news = increaseIndex(_oldest_news); // but make sure we're not overflowing here
+	/* If the fifo-buffer is full, overwrite the oldest entry */
+	if (l_news != INVALID_NEWS && _latest_news == _oldest_news) {
+		assert(_total_news == MAX_NEWS);
+		_oldest_news = increaseIndex(_oldest_news);
+	}
 
-	// add news to _latest_news
-	ni = &_news_items[_latest_news];
-	memset(ni, 0, sizeof(*ni));
+	/*DEBUG(misc, 0) ("+cur %3d, old %2d, lat %3d, for %3d, tot %2d",
+	  _current_news, _oldest_news, _latest_news, _forced_news, _total_news); */
 
-	ni->string_id = string;
-	ni->display_mode = (byte)flags;
-	ni->flags = (byte)(flags >> 8) | NF_NOEXPIRE;
+	{ /* Add news to _latest_news */
+		Window *w;
+		NewsItem *ni = &_news_items[_latest_news];
+		memset(ni, 0, sizeof(*ni));
 
-	// show this news message in color?
-	if (_date >= ConvertIntDate(_patches.colored_news_date))
-		ni->flags |= NF_INCOLOR;
+		ni->string_id = string;
+		ni->display_mode = (byte)flags;
+		ni->flags = (byte)(flags >> 8);
 
-	ni->type = (byte)(flags >> 16);
-	ni->callback = (byte)(flags >> 24);
-	ni->data_a = data_a;
-	ni->data_b = data_b;
-	ni->date = _date;
-	COPY_OUT_DPARAM(ni->params, 0, lengthof(ni->params));
+		// show this news message in color?
+		if (_cur_year >= _patches.colored_news_year) ni->flags |= NF_INCOLOR;
 
-	w = FindWindowById(WC_MESSAGE_HISTORY, 0);
-	if (w == NULL) return;
-	SetWindowDirty(w);
-	w->vscroll.count = _total_news;
+		ni->type = (byte)(flags >> 16);
+		ni->callback = (byte)(flags >> 24);
+		ni->data_a = data_a;
+		ni->data_b = data_b;
+		ni->date = _date;
+		COPY_OUT_DPARAM(ni->params, 0, lengthof(ni->params));
+
+		w = FindWindowById(WC_MESSAGE_HISTORY, 0);
+		if (w == NULL) return;
+		SetWindowDirty(w);
+		w->vscroll.count = _total_news;
+	}
 }
 
-/* To add a news item with an attached validation function. This validation function
- * makes sure that the news item is not outdated when the newspaper pops up. */
-void AddValidatedNewsItem(StringID string, uint32 flags, uint data_a, uint data_b, ValidationProc *validation)
-{
-	AddNewsItem(string, flags, data_a, data_b);
-	_news_items[_latest_news].isValid = validation;
-}
 
-// don't show item if it's older than x days
+/* Don't show item if it's older than x days, corresponds with NewsType in news.h */
 static const byte _news_items_age[] = {60, 60, 90, 60, 90, 30, 150, 30, 90, 180};
 
 static const Widget _news_type13_widgets[] = {
@@ -316,10 +337,10 @@ static WindowDesc _news_type2_desc = {
 };
 
 static const Widget _news_type0_widgets[] = {
-{      WWT_PANEL,   RESIZE_NONE,     5,     0,   279,    14,    86, 0x0,								STR_NULL},
-{   WWT_CLOSEBOX,   RESIZE_NONE,     5,     0,    10,     0,    13, STR_00C5,					STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,   RESIZE_NONE,     5,    11,   279,     0,    13, STR_012C_MESSAGE,	STR_NULL},
-{          WWT_6,   RESIZE_NONE,     5,     2,   277,    16,    64, 0x0,								STR_NULL},
+{      WWT_PANEL,   RESIZE_NONE,     5,     0,   279,    14,    86, 0x0,              STR_NULL},
+{   WWT_CLOSEBOX,   RESIZE_NONE,     5,     0,    10,     0,    13, STR_00C5,         STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,   RESIZE_NONE,     5,    11,   279,     0,    13, STR_012C_MESSAGE, STR_NULL},
+{      WWT_INSET,   RESIZE_NONE,     5,     2,   277,    16,    64, 0x0,              STR_NULL},
 {   WIDGETS_END},
 };
 
@@ -331,19 +352,34 @@ static WindowDesc _news_type0_desc = {
 	NewsWindowProc
 };
 
-static const SoundFx _news_sounds[] = {
+static const SoundFx _news_sounds[NT_END] = {
 	SND_1D_APPLAUSE,
 	SND_1D_APPLAUSE,
 	0,
 	0,
 	0,
 	0,
+	0,
 	SND_1E_OOOOH,
 	0,
 	0,
 	0
 };
 
+const char *_news_display_name[NT_END] = {
+	"arrival_player",
+	"arrival_other",
+	"accident",
+	"company_info",
+	"openclose",
+	"economy",
+	"advice",
+	"new_vehicles",
+	"acceptance",
+	"subsidies",
+	"general",
+};
+
 /** Get the value of an item of the news-display settings. This is
  * a little tricky since on/off/summary must use 2 bits to store the value
  * @param item the item whose value is requested
@@ -351,7 +387,7 @@ static const SoundFx _news_sounds[] = {
  */
 static inline byte GetNewsDisplayValue(byte item)
 {
-	assert(item < 10 && GB(_news_display_opt, item * 2, 2) <= 2);
+	assert(item < NT_END && GB(_news_display_opt, item * 2, 2) <= 2);
 	return GB(_news_display_opt, item * 2, 2);
 }
 
@@ -362,7 +398,7 @@ static inline byte GetNewsDisplayValue(byte item)
  */
 static inline void SetNewsDisplayValue(byte item, byte val)
 {
-	assert(item < 10 && val <= 2);
+	assert(item < NT_END && val <= 2);
 	SB(_news_display_opt, item * 2, 2, val);
 }
 
@@ -372,7 +408,7 @@ static void ShowNewspaper(NewsItem *ni)
 	Window *w;
 	SoundFx sound;
 	int top;
-	ni->flags &= ~(NF_NOEXPIRE | NF_FORCE_BIG);
+	ni->flags &= ~NF_FORCE_BIG;
 	ni->duration = 555;
 
 	sound = _news_sounds[ni->type];
@@ -408,6 +444,10 @@ static void ShowNewspaper(NewsItem *ni)
 			break;
 		}
 	}
+
+	/*DEBUG(misc, 0) (" cur %3d, old %2d, lat %3d, for %3d, tot %2d",
+	  _current_news, _oldest_news, _latest_news, _forced_news, _total_news); */
+
 	WP(w, news_d).ni = &_news_items[_forced_news == INVALID_NEWS ? _current_news : _forced_news];
 	w->flags4 |= WF_DISABLE_VP_SCROLL;
 }
@@ -430,7 +470,7 @@ static void ShowTicker(const NewsItem *ni)
 static bool ReadyForNextItem(void)
 {
 	const Window *w;
-	byte item = _forced_news == INVALID_NEWS ? _current_news : _forced_news;
+	NewsID item = (_forced_news == INVALID_NEWS) ? _current_news : _forced_news;
 	NewsItem *ni;
 
 	if (item >= MAX_NEWS) return true;
@@ -441,38 +481,31 @@ static bool ReadyForNextItem(void)
 	w = FindWindowById(WC_STATUS_BAR, 0);
 	if (w != NULL && WP(w, const def_d).data_1 > -1280) return false;
 
-	// Newspaper message
-	// Wait until duration reaches 0
-	if (ni->duration != 0) {
-		ni->duration--;
-		return false;
-	}
+	// Newspaper message, decrement duration counter
+	if (ni->duration != 0) ni->duration--;
 
 	// neither newsticker nor newspaper are running
-	return true;
+	return (ni->duration == 0 || FindWindowById(WC_NEWS_WINDOW, 0) == NULL);
 }
 
-static void MoveToNexItem(void)
+static void MoveToNextItem(void)
 {
 	DeleteWindowById(WC_NEWS_WINDOW, 0);
 	_forced_news = INVALID_NEWS;
 
-	// if we're not at the last item, than move on
+	// if we're not at the last item, then move on
 	if (_current_news != _latest_news) {
 		NewsItem *ni;
 
-		_current_news = increaseIndex(_current_news);
+		_current_news = (_current_news == INVALID_NEWS) ? _oldest_news : increaseIndex(_current_news);
 		ni = &_news_items[_current_news];
 
 		// check the date, don't show too old items
 		if (_date - _news_items_age[ni->type] > ni->date) return;
 
-		// execute the validation function to see if this item is still valid
-		if (ni->isValid != NULL && !ni->isValid(ni->data_a, ni->data_b)) return;
-
 		switch (GetNewsDisplayValue(ni->type)) {
 		case 0: { /* Off - show nothing only a small reminder in the status bar */
-			Window* w = FindWindowById(WC_STATUS_BAR, 0);
+			Window *w = FindWindowById(WC_STATUS_BAR, 0);
 
 			if (w != NULL) {
 				WP(w, def_d).data_2 = 91;
@@ -500,11 +533,11 @@ void NewsLoop(void)
 	// no news item yet
 	if (_total_news == 0) return;
 
-	if (ReadyForNextItem()) MoveToNexItem();
+	if (ReadyForNextItem()) MoveToNextItem();
 }
 
 /* Do a forced show of a specific message */
-static void ShowNewsMessage(byte i)
+static void ShowNewsMessage(NewsID i)
 {
 	if (_total_news == 0) return;
 
@@ -517,7 +550,7 @@ static void ShowNewsMessage(byte i)
 	if (_forced_news != INVALID_NEWS) {
 		NewsItem *ni = &_news_items[_forced_news];
 		ni->duration = 555;
-		ni->flags |= NF_NOEXPIRE | NF_FORCE_BIG;
+		ni->flags |= NF_FORCE_BIG;
 		DeleteWindowById(WC_NEWS_WINDOW, 0);
 		ShowNewspaper(ni);
 	}
@@ -526,18 +559,23 @@ static void ShowNewsMessage(byte i)
 void ShowLastNewsMessage(void)
 {
 	if (_forced_news == INVALID_NEWS) {
-		ShowNewsMessage(_current_news);
-	} else if (_forced_news != 0) {
-		ShowNewsMessage(_forced_news - 1);
+		/* Not forced any news yet, show the current one, unless a news window is
+		 * open (which can only be the current one), then show the previous item */
+		const Window *w = FindWindowById(WC_NEWS_WINDOW, 0);
+		ShowNewsMessage((w == NULL || (_current_news == _oldest_news)) ? _current_news : decreaseIndex(_current_news));
+	} else if (_forced_news == _oldest_news) {
+		/* We have reached the oldest news, start anew with the latest */
+		ShowNewsMessage(_latest_news);
 	} else {
-		ShowNewsMessage(_total_news != MAX_NEWS ? _latest_news : MAX_NEWS - 1);
+		/* 'Scrolling' through news history show each one in turn */
+		ShowNewsMessage(decreaseIndex(_forced_news));
 	}
 }
 
 
 /* return news by number, with 0 being the most
-recent news. Returns INVALID_NEWS if end of queue reached. */
-static byte getNews(byte i)
+ * recent news. Returns INVALID_NEWS if end of queue reached. */
+static NewsID getNews(NewsID i)
 {
 	if (i >= _total_news) return INVALID_NEWS;
 
@@ -561,8 +599,10 @@ static byte getNews(byte i)
 static void DrawNewsString(int x, int y, uint16 color, const NewsItem *ni, uint maxw)
 {
 	char buffer[512], buffer2[512];
-	char *ptr, *dest;
+	const char *ptr;
+	char *dest;
 	StringID str;
+	WChar c_last;
 
 	if (ni->display_mode == 3) {
 		str = _get_news_string_callback[ni->callback](ni);
@@ -571,16 +611,26 @@ static void DrawNewsString(int x, int y, uint16 color, const NewsItem *ni, uint
 		str = ni->string_id;
 	}
 
-	GetString(buffer, str);
+	GetString(buffer, str, lastof(buffer));
 	/* Copy the just gotten string to another buffer to remove any formatting
 	 * from it such as big fonts, etc. */
-	for (ptr = buffer, dest = buffer2; *ptr != '\0'; ptr++) {
-		if (*ptr == '\r') {
+	ptr  = buffer;
+	dest = buffer2;
+	c_last = '\0';
+	for (;;) {
+		WChar c = Utf8Consume(&ptr);
+		if (c == 0) break;
+		/* Make a space from a newline, but ignore multiple newlines */
+		if (c == '\n' && c_last != '\n') {
+			dest[0] = ' ';
+			dest++;
+		} else if (c == '\r') {
 			dest[0] = dest[1] = dest[2] = dest[3] = ' ';
 			dest += 4;
-		} else if ((byte)*ptr >= ' ' && ((byte)*ptr < 0x88 || (byte)*ptr >= 0x99)) {
-			*dest++ = *ptr;
+		} else if (IsPrintable(c)) {
+			dest += Utf8Encode(dest, c);
 		}
+		c_last = c;
 	}
 
 	*dest = '\0';
@@ -594,7 +644,7 @@ static void MessageHistoryWndProc(Window *w, WindowEvent *e)
 	switch (e->event) {
 	case WE_PAINT: {
 		int y = 19;
-		byte p, show;
+		NewsID p, show;
 
 		SetVScrollCount(w, _total_news);
 		DrawWindowWidgets(w);
@@ -616,50 +666,32 @@ static void MessageHistoryWndProc(Window *w, WindowEvent *e)
 	}
 
 	case WE_CLICK:
-		switch (e->click.widget) {
+		switch (e->we.click.widget) {
 		case 3: {
-			int y = (e->click.pt.y - 19) / 12;
-			byte p, q;
+			int y = (e->we.click.pt.y - 19) / 12;
+			NewsID p = getNews(y + w->vscroll.pos);
 
-			#if 0 // === DEBUG code only
-			for (p = 0; p < _total_news; p++) {
-				NewsItem *ni;
-				byte buffer[256];
-				ni = &_news_items[p];
-				GetNewsString(ni, buffer);
-				printf("%i\t%i\t%s\n", p, ni->date, buffer);
-			}
-			printf("=========================\n");
-			#endif
-
-			p = y + w->vscroll.pos;
-			if (p > _total_news - 1) break;
-
-			if (_latest_news >= p) {
-				q = _latest_news - p;
-			} else {
-				q = _latest_news + MAX_NEWS - p;
-			}
-			ShowNewsMessage(q);
+			if (p == INVALID_NEWS) break;
 
+			ShowNewsMessage(p);
 			break;
 		}
 		}
 		break;
 
 	case WE_RESIZE:
-		w->vscroll.cap += e->sizing.diff.y / 12;
+		w->vscroll.cap += e->we.sizing.diff.y / 12;
 		break;
 	}
 }
 
 static const Widget _message_history_widgets[] = {
-{   WWT_CLOSEBOX,   RESIZE_NONE,    13,     0,    10,     0,    13, STR_00C5,			STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,  RESIZE_RIGHT,    13,    11,   387,     0,    13, STR_MESSAGE_HISTORY,	STR_018C_WINDOW_TITLE_DRAG_THIS},
-{  WWT_STICKYBOX,     RESIZE_LR,    13,   388,   399,     0,    13, 0x0,										STR_STICKY_BUTTON},
-{     WWT_IMGBTN,     RESIZE_RB,    13,     0,   387,    14,   139, 0x0, STR_MESSAGE_HISTORY_TIP},
-{  WWT_SCROLLBAR,    RESIZE_LRB,    13,   388,   399,    14,   127, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
-{  WWT_RESIZEBOX,   RESIZE_LRTB,    13,   388,   399,   128,   139, 0x0, STR_RESIZE_BUTTON},
+{   WWT_CLOSEBOX,   RESIZE_NONE,    13,     0,    10,     0,    13, STR_00C5,            STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,  RESIZE_RIGHT,    13,    11,   387,     0,    13, STR_MESSAGE_HISTORY, STR_018C_WINDOW_TITLE_DRAG_THIS},
+{  WWT_STICKYBOX,     RESIZE_LR,    13,   388,   399,     0,    13, 0x0,                 STR_STICKY_BUTTON},
+{      WWT_PANEL,     RESIZE_RB,    13,     0,   387,    14,   139, 0x0,                 STR_MESSAGE_HISTORY_TIP},
+{  WWT_SCROLLBAR,    RESIZE_LRB,    13,   388,   399,    14,   127, 0x0,                 STR_0190_SCROLL_BAR_SCROLLS_LIST},
+{  WWT_RESIZEBOX,   RESIZE_LRTB,    13,   388,   399,   128,   139, 0x0,                 STR_RESIZE_BUTTON},
 {   WIDGETS_END},
 };
 
@@ -697,43 +729,33 @@ void ShowMessageHistory(void)
 static void SetMessageButtonStates(Window *w, byte value, int element)
 {
 	element *= 2;
-	switch (value) {
-	case 0: /* Off */
-		SETBIT(w->disabled_state, element + 3);
-		CLRBIT(w->disabled_state, element + 3 + 1);
-		break;
-	case 1: /* Summary */
-		CLRBIT(w->disabled_state, element + 3);
-		CLRBIT(w->disabled_state, element + 3 + 1);
-		break;
-	case 2: /* Full */
-		SETBIT(w->disabled_state, element + 3 + 1);
-		CLRBIT(w->disabled_state, element + 3);
-		break;
-	default: NOT_REACHED();
-	}
+
+	SetWindowWidgetDisabledState(w, element + 3, value == 0);
+	SetWindowWidgetDisabledState(w, element + 3 + 1, value == 2);
 }
 
 static void MessageOptionsWndProc(Window *w, WindowEvent *e)
 {
 	static const StringID message_opt[] = {STR_OFF, STR_SUMMARY, STR_FULL, INVALID_STRING_ID};
-	static const uint32 message_val[] = {0x0, 0x55555555, 0xAAAAAAAA}; // 0x555.. = 01010101010101010101 (all summary), 286.. 1010... (full)
-	static const uint32 message_dis[] = {
-		(1 << 3) | (1 << 5) | (1 << 7) | (1 << 9)  | (1 << 11) | (1 << 13) | (1 << 15) | (1 << 17) | (1 << 19) | (1 << 21),
-		0,
-		(1 << 4) | (1 << 6) | (1 << 8) | (1 << 10) | (1 << 12) | (1 << 14) | (1 << 16) | (1 << 18) | (1 << 20) | (1 << 22),
-	};
 
 	/* WP(w, def_d).data_1 are stores the clicked state of the fake widgets
 	 * WP(w, def_d).data_2 stores state of the ALL on/off/summary button */
 	switch (e->event) {
 	case WE_CREATE: {
 		uint32 val = _news_display_opt;
+		uint32 all_val;
 		int i;
-		WP(w, def_d).data_1 = WP(w, def_d).data_2 = 0;
+		WP(w, def_d).data_1 = 0;
 
-		// Set up the initial disabled buttons in the case of 'off' or 'full'
-		for (i = 0; i != 10; i++, val >>= 2) SetMessageButtonStates(w, val & 0x3, i);
+		/* Set up the initial disabled buttons in the case of 'off' or 'full' */
+		all_val = val & 0x3;
+		for (i = 0; i != 10; i++, val >>= 2) {
+			SetMessageButtonStates(w, val & 0x3, i);
+			/* If the value doesn't match the ALL-button value, set the ALL-button value to 'off' */
+			if ((val & 0x3) != all_val) all_val = 0;
+		}
+		/* If all values are the same value, the ALL-button will take over this value */
+		WP(w, def_d).data_2 = all_val;
 	} break;
 
 	case WE_PAINT: {
@@ -741,13 +763,12 @@ static void MessageOptionsWndProc(Window *w, WindowEvent *e)
 		int click_state = WP(w, def_d).data_1;
 		int i, y;
 
-		if (_news_ticker_sound) SETBIT(w->click_state, 25);
+		if (_news_ticker_sound) LowerWindowWidget(w, 27);
 		DrawWindowWidgets(w);
-		DrawStringCentered(185, 15, STR_0205_MESSAGE_TYPES, 0);
 
 		/* XXX - Draw the fake widgets-buttons. Can't add these to the widget-desc since
 		 * openttd currently can only handle 32 widgets. So hack it *g* */
-		for (i = 0, y = 26; i != 10; i++, y += 12, click_state >>= 1, val >>= 2) {
+		for (i = 0, y = 26; i < NT_END; i++, y += 12, click_state >>= 1, val >>= 2) {
 			bool clicked = !!(click_state & 1);
 
 			DrawFrameRect(13, y, 89, 11 + y, 3, (clicked) ? FR_LOWERED : 0);
@@ -755,17 +776,17 @@ static void MessageOptionsWndProc(Window *w, WindowEvent *e)
 			DrawString(103, y + 1, i + STR_0206_ARRIVAL_OF_FIRST_VEHICLE, 0);
 		}
 
-		DrawString(  8, y + 9, message_opt[WP(w, def_d).data_2], 0x10);
+		DrawStringCentered(((13 + 89 + 1) >> 1), y + 9, message_opt[WP(w, def_d).data_2], 0x10);
 		DrawString(103, y + 9, STR_MESSAGES_ALL, 0);
 		DrawString(103, y + 9 + 12, STR_MESSAGE_SOUND, 0);
 
 	} break;
 
 	case WE_CLICK:
-		switch (e->click.widget) {
+		switch (e->we.click.widget) {
 		case 2: /* Clicked on any of the fake widgets */
-			if (e->click.pt.x > 13 && e->click.pt.x < 89 && e->click.pt.y > 26 && e->click.pt.y < 146) {
-				int element = (e->click.pt.y - 26) / 12;
+			if (e->we.click.pt.x > 13 && e->we.click.pt.x < 89 && e->we.click.pt.y > 26 && e->we.click.pt.y < NT_END * 12 + 26) {
+				int element = (e->we.click.pt.y - 26) / 12;
 				byte val = (GetNewsDisplayValue(element) + 1) % 3;
 
 				SetMessageButtonStates(w, val, element);
@@ -776,17 +797,17 @@ static void MessageOptionsWndProc(Window *w, WindowEvent *e)
 				SetWindowDirty(w);
 			}
 			break;
-		case 23: case 24: /* Dropdown menu for all settings */
-			ShowDropDownMenu(w, message_opt, WP(w, def_d).data_2, 24, 0, 0);
+		case 25: case 26: /* Dropdown menu for all settings */
+			ShowDropDownMenu(w, message_opt, WP(w, def_d).data_2, 26, 0, 0);
 			break;
-		case 25: /* Change ticker sound on/off */
+		case 27: /* Change ticker sound on/off */
 			_news_ticker_sound ^= 1;
-			TOGGLEBIT(w->click_state, e->click.widget);
-			InvalidateWidget(w, e->click.widget);
+			ToggleWidgetLoweredState(w, e->we.click.widget);
+			InvalidateWidget(w, e->we.click.widget);
 			break;
 		default: { /* Clicked on the [<] .. [>] widgets */
-			int wid = e->click.widget;
-			if (wid > 2 && wid < 23) {
+			int wid = e->we.click.widget;
+			if (wid > 2 && wid <= NT_END * 2 + 2) {
 				int element = (wid - 3) / 2;
 				byte val = (GetNewsDisplayValue(element) + ((wid & 1) ? -1 : 1)) % 3;
 
@@ -797,65 +818,74 @@ static void MessageOptionsWndProc(Window *w, WindowEvent *e)
 		} break;
 		} break;
 
-	case WE_DROPDOWN_SELECT: /* Select all settings for newsmessages */
-		WP(w, def_d).data_2 = e->dropdown.index;
-		_news_display_opt = message_val[WP(w, def_d).data_2];
-		w->disabled_state = message_dis[WP(w, def_d).data_2];
+	case WE_DROPDOWN_SELECT: {/* Select all settings for newsmessages */
+		int i;
+
+		WP(w, def_d).data_2 = e->we.dropdown.index;
+
+		for (i = 0; i < NT_END; i++) {
+			SetMessageButtonStates(w, e->we.dropdown.index, i);
+			SetNewsDisplayValue(i, e->we.dropdown.index);
+		}
 		SetWindowDirty(w);
 		break;
+		}
 
 	case WE_TIMEOUT: /* XXX - Hack to animate 'fake' buttons */
 		WP(w, def_d).data_1 = 0;
 		SetWindowDirty(w);
 		break;
-
 	}
 }
 
 static const Widget _message_options_widgets[] = {
-{   WWT_CLOSEBOX,   RESIZE_NONE,    13,     0,   10,     0,    13, STR_00C5,             STR_018B_CLOSE_WINDOW},
+{   WWT_CLOSEBOX,   RESIZE_NONE,    13,     0,   10,     0,    13, STR_00C5,                 STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,   RESIZE_NONE,    13,    11,  409,     0,    13, STR_0204_MESSAGE_OPTIONS, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{      WWT_PANEL,   RESIZE_NONE,    13,     0,  409,    14,   184, STR_NULL,             STR_NULL},
+{      WWT_PANEL,   RESIZE_NONE,    13,     0,  409,    14,   196, 0x0,                      STR_NULL},
 
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    26,    37, SPR_ARROW_LEFT,       STR_HSCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    26,    37, SPR_ARROW_RIGHT,      STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    26,    37, SPR_ARROW_LEFT,           STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    26,    37, SPR_ARROW_RIGHT,          STR_HSCROLL_BAR_SCROLLS_LIST},
 
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    38,    49, SPR_ARROW_LEFT,       STR_HSCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    38,    49, SPR_ARROW_RIGHT,      STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    38,    49, SPR_ARROW_LEFT,           STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    38,    49, SPR_ARROW_RIGHT,          STR_HSCROLL_BAR_SCROLLS_LIST},
 
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    50,    61, SPR_ARROW_LEFT,       STR_HSCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    50,    61, SPR_ARROW_RIGHT,      STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    50,    61, SPR_ARROW_LEFT,           STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    50,    61, SPR_ARROW_RIGHT,          STR_HSCROLL_BAR_SCROLLS_LIST},
 
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    62,    73, SPR_ARROW_LEFT,       STR_HSCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    62,    73, SPR_ARROW_RIGHT,      STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    62,    73, SPR_ARROW_LEFT,           STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    62,    73, SPR_ARROW_RIGHT,          STR_HSCROLL_BAR_SCROLLS_LIST},
 
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    74,    85, SPR_ARROW_LEFT,       STR_HSCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    74,    85, SPR_ARROW_RIGHT,      STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    74,    85, SPR_ARROW_LEFT,           STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    74,    85, SPR_ARROW_RIGHT,          STR_HSCROLL_BAR_SCROLLS_LIST},
 
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    86,    97, SPR_ARROW_LEFT,       STR_HSCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    86,    97, SPR_ARROW_RIGHT,      STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    86,    97, SPR_ARROW_LEFT,           STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    86,    97, SPR_ARROW_RIGHT,          STR_HSCROLL_BAR_SCROLLS_LIST},
 
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    98,   109, SPR_ARROW_LEFT,       STR_HSCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    98,   109, SPR_ARROW_RIGHT,      STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,    98,   109, SPR_ARROW_LEFT,           STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,    98,   109, SPR_ARROW_RIGHT,          STR_HSCROLL_BAR_SCROLLS_LIST},
 
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,   110,   121, SPR_ARROW_LEFT,       STR_HSCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,   110,   121, SPR_ARROW_RIGHT,      STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,   110,   121, SPR_ARROW_LEFT,           STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,   110,   121, SPR_ARROW_RIGHT,          STR_HSCROLL_BAR_SCROLLS_LIST},
 
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,   122,   133, SPR_ARROW_LEFT,       STR_HSCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,   122,   133, SPR_ARROW_RIGHT,      STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,   122,   133, SPR_ARROW_LEFT,           STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,   122,   133, SPR_ARROW_RIGHT,          STR_HSCROLL_BAR_SCROLLS_LIST},
 
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,   134,   145, SPR_ARROW_LEFT,       STR_HSCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,   134,   145, SPR_ARROW_RIGHT,      STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,   134,   145, SPR_ARROW_LEFT,           STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,   134,   145, SPR_ARROW_RIGHT,          STR_HSCROLL_BAR_SCROLLS_LIST},
 
-{      WWT_PANEL,   RESIZE_NONE,     3,     4,   86,   154,   165, STR_NULL,             STR_NULL},
-{    WWT_TEXTBTN,   RESIZE_NONE,     3,    87,   98,   154,   165, STR_0225,             STR_NULL},
-{          WWT_4,   RESIZE_NONE,     3,     4,   98,   166,   177, STR_02DB_OFF,         STR_NULL},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,     4,   12,   146,   157, SPR_ARROW_LEFT,           STR_HSCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHIMGBTN,   RESIZE_NONE,     3,    90,   98,   146,   157, SPR_ARROW_RIGHT,          STR_HSCROLL_BAR_SCROLLS_LIST},
 
+{      WWT_PANEL,   RESIZE_NONE,     3,     4,   86,   166,   177, 0x0,                      STR_NULL},
+{    WWT_TEXTBTN,   RESIZE_NONE,     3,    87,   98,   166,   177, STR_0225,                 STR_NULL},
+{  WWT_TEXTBTN_2,   RESIZE_NONE,     3,     4,   98,   178,   189, STR_02DB_OFF,             STR_NULL},
+
+{      WWT_LABEL,   RESIZE_NONE,    13,     0,  409,    13,    26, STR_0205_MESSAGE_TYPES,   STR_NULL},
 {   WIDGETS_END},
 };
 
 static const WindowDesc _message_options_desc = {
-	270, 22, 410, 185,
+	270, 22, 410, 197,
 	WC_GAME_OPTIONS, 0,
 	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS,
 	_message_options_widgets,
@@ -867,3 +897,70 @@ void ShowMessageOptions(void)
 	DeleteWindowById(WC_GAME_OPTIONS, 0);
 	AllocateWindowDesc(&_message_options_desc);
 }
+
+
+void DeleteVehicleNews(VehicleID vid, StringID news)
+{
+	NewsID n;
+
+	for (n = _oldest_news; _latest_news != INVALID_NEWS; n = increaseIndex(n)) {
+		const NewsItem *ni = &_news_items[n];
+
+		if (ni->flags & NF_VEHICLE &&
+				ni->data_a == vid &&
+				(news == INVALID_STRING_ID || ni->string_id == news)) {
+			Window *w;
+
+			/* If we delete a forced news and it is just before the current news
+			 * then we need to advance to the next news (if any) */
+			if (_forced_news == n) MoveToNextItem();
+			if (_forced_news == INVALID_NEWS && _current_news == n) MoveToNextItem();
+			_total_news--;
+
+			/* If this is the last news item, invalidate _latest_news */
+			if (_total_news == 0) {
+				assert(_latest_news == _oldest_news);
+				_latest_news = INVALID_NEWS;
+			}
+
+			/* Since we only imitate a FIFO removing an arbitrary element does need
+			 * some magic. Remove the item by shifting head towards the tail. eg
+			 *    oldest    remove  last
+			 *        |        |     |
+			 * [------O--------n-----L--]
+			 * will become (change dramatized to make clear)
+			 * [---------O-----------L--]
+			 * We also need an update of the current, forced and visible (open window)
+			 * news's as this shifting could change the items they were pointing to */
+			if (_total_news != 0) {
+				NewsID visible_news, i;
+				w = FindWindowById(WC_NEWS_WINDOW, 0);
+				visible_news = (w != NULL) ? (NewsID)(WP(w, news_d).ni - _news_items) : INVALID_NEWS;
+
+				for (i = n;; i = decreaseIndex(i)) {
+					_news_items[i] = _news_items[decreaseIndex(i)];
+
+					if (i != _latest_news) {
+						if (i == _current_news) _current_news = increaseIndex(_current_news);
+						if (i == _forced_news) _forced_news = increaseIndex(_forced_news);
+						if (i == visible_news) WP(w, news_d).ni = &_news_items[increaseIndex(visible_news)];
+					}
+
+					if (i == _oldest_news) break;
+				}
+				_oldest_news = increaseIndex(_oldest_news);
+			}
+
+			/*DEBUG(misc, 0) ("-cur %3d, old %2d, lat %3d, for %3d, tot %2d",
+			  _current_news, _oldest_news, _latest_news, _forced_news, _total_news); */
+
+			w = FindWindowById(WC_MESSAGE_HISTORY, 0);
+			if (w != NULL) {
+				SetWindowDirty(w);
+				w->vscroll.count = _total_news;
+			}
+		}
+
+		if (n == _latest_news) break;
+	}
+}
