@@ -1,7 +1,8 @@
-/* $Id: npf.c 4072 2006-03-23 17:52:08Z matthijs $ */
+/* $Id: npf.c 10841 2007-08-10 17:53:12Z rubidium $ */
 
 #include "stdafx.h"
 #include "openttd.h"
+#include "bridge_map.h"
 #include "debug.h"
 #include "functions.h"
 #include "npf.h"
@@ -9,8 +10,12 @@
 #include "macros.h"
 #include "pathfind.h"
 #include "station.h"
+#include "station_map.h"
 #include "tile.h"
 #include "depot.h"
+#include "tunnel_map.h"
+#include "network.h"
+#include "water_map.h"
 
 static AyStar _npf_aystar;
 
@@ -47,6 +52,7 @@ static uint NPFDistanceTrack(TileIndex t0, TileIndex t1)
 	return diagTracks * NPF_TILE_LENGTH + straightTracks * NPF_TILE_LENGTH * STRAIGHT_TRACK_LENGTH;
 }
 
+
 #if 0
 static uint NTPHash(uint key1, uint key2)
 {
@@ -57,10 +63,10 @@ static uint NTPHash(uint key1, uint key2)
 
 /**
  * Calculates a hash value for use in the NPF.
- * @param key1	The TileIndex of the tile to hash
- * @param key1	The Trackdir of the track on the tile.
+ * @param key1 The TileIndex of the tile to hash
+ * @param key2 The Trackdir of the track on the tile.
  *
- * @todo	Think of a better hash.
+ * @todo Think of a better hash.
  */
 static uint NPFHash(uint key1, uint key2)
 {
@@ -70,7 +76,7 @@ static uint NPFHash(uint key1, uint key2)
 
 	assert(IsValidTrackdir(key2));
 	assert(IsValidTile(key1));
-	return ((((part1 << NPF_HASH_HALFBITS) | part2)) + (NPF_HASH_SIZE * key2 / TRACKDIR_END)) % NPF_HASH_SIZE;
+	return ((part1 << NPF_HASH_HALFBITS | part2) + (NPF_HASH_SIZE * key2 / TRACKDIR_END)) % NPF_HASH_SIZE;
 }
 
 static int32 NPFCalcZero(AyStar* as, AyStarNode* current, OpenListNode* parent)
@@ -116,15 +122,16 @@ static int32 NPFCalcStationOrTileHeuristic(AyStar* as, AyStarNode* current, Open
 	uint dist;
 
 	// for train-stations, we are going to aim for the closest station tile
-	if ((as->user_data[NPF_TYPE] == TRANSPORT_RAIL) && (fstd->station_index != -1))
+	if (as->user_data[NPF_TYPE] == TRANSPORT_RAIL && fstd->station_index != INVALID_STATION)
 		to = CalcClosestStationTile(fstd->station_index, from);
 
-	if (as->user_data[NPF_TYPE] == TRANSPORT_ROAD)
+	if (as->user_data[NPF_TYPE] == TRANSPORT_ROAD) {
 		/* Since roads only have diagonal pieces, we use manhattan distance here */
 		dist = DistanceManhattan(from, to) * NPF_TILE_LENGTH;
-	else
+	} else {
 		/* Ships and trains can also go diagonal, so the minimum distance is shorter */
 		dist = NPFDistanceTrack(from, to);
+	}
 
 	DEBUG(npf, 4)("Calculating H for: (%d, %d). Result: %d", TileX(current->tile), TileY(current->tile), dist);
 
@@ -135,6 +142,7 @@ static int32 NPFCalcStationOrTileHeuristic(AyStar* as, AyStarNode* current, Open
 	return dist;
 }
 
+
 /* Fills AyStarNode.user_data[NPF_TRACKDIRCHOICE] with the chosen direction to
  * get here, either getting it from the current choice or from the parent's
  * choice */
@@ -145,13 +153,11 @@ static void NPFFillTrackdirChoice(AyStarNode* current, OpenListNode* parent)
 		/* This is a first order decision, so we'd better save the
 		 * direction we chose */
 		current->user_data[NPF_TRACKDIR_CHOICE] = trackdir;
-		DEBUG(npf, 6)("Saving trackdir: %#x", trackdir);
+		DEBUG(npf, 6)("Saving trackdir: 0x%X", trackdir);
 	} else {
-		/* We've already made the decision, so just save our parent's
-		 * decision */
+		/* We've already made the decision, so just save our parent's decision */
 		current->user_data[NPF_TRACKDIR_CHOICE] = parent->path.node.user_data[NPF_TRACKDIR_CHOICE];
 	}
-
 }
 
 /* Will return the cost of the tunnel. If it is an entry, it will return the
@@ -161,11 +167,11 @@ static uint NPFTunnelCost(AyStarNode* current)
 {
 	DiagDirection exitdir = TrackdirToExitdir((Trackdir)current->direction);
 	TileIndex tile = current->tile;
-	if ((DiagDirection)GB(_m[tile].m5, 0, 2) == ReverseDiagdir(exitdir)) {
+	if (GetTunnelDirection(tile) == ReverseDiagDir(exitdir)) {
 		/* We just popped out if this tunnel, since were
 		 * facing the tunnel exit */
 		FindLengthOfTunnelResult flotr;
-		flotr = FindLengthOfTunnel(tile, ReverseDiagdir(exitdir));
+		flotr = FindLengthOfTunnel(tile, ReverseDiagDir(exitdir));
 		return flotr.length * NPF_TILE_LENGTH;
 		//TODO: Penalty for tunnels?
 	} else {
@@ -177,21 +183,21 @@ static uint NPFTunnelCost(AyStarNode* current)
 
 static uint NPFSlopeCost(AyStarNode* current)
 {
-	TileIndex next = current->tile + TileOffsByDir(TrackdirToExitdir(current->direction));
+	TileIndex next = current->tile + TileOffsByDiagDir(TrackdirToExitdir(current->direction));
 	int x,y;
 	int8 z1,z2;
 
 	x = TileX(current->tile) * TILE_SIZE;
 	y = TileY(current->tile) * TILE_SIZE;
 	/* get the height of the center of the current tile */
-	z1 = GetSlopeZ(x+TILE_HEIGHT, y+TILE_HEIGHT);
+	z1 = GetSlopeZ(x + TILE_SIZE / 2, y + TILE_SIZE / 2);
 
 	x = TileX(next) * TILE_SIZE;
 	y = TileY(next) * TILE_SIZE;
 	/* get the height of the center of the next tile */
-	z2 = GetSlopeZ(x+TILE_HEIGHT, y+TILE_HEIGHT);
+	z2 = GetSlopeZ(x + TILE_SIZE / 2, y + TILE_SIZE / 2);
 
-	if ((z2 - z1) > 1) {
+	if (z2 - z1 > 1) {
 		/* Slope up */
 		return _patches.npf_rail_slope_penalty;
 	}
@@ -201,31 +207,33 @@ static uint NPFSlopeCost(AyStarNode* current)
 	 * there is only one level of steepness... */
 }
 
-/* Mark tiles by mowing the grass when npf debug level >= 1 */
+/**
+ * Mark tiles by mowing the grass when npf debug level >= 1.
+ * Will not work for multiplayer games, since it can (will) cause desyncs.
+ */
 static void NPFMarkTile(TileIndex tile)
 {
-#ifdef NO_DEBUG_MESSAGES
-	return;
-#else
-	if (_debug_npf_level >= 1)
-		switch(GetTileType(tile)) {
-			case MP_RAILWAY:
-				/* DEBUG: mark visited tiles by mowing the grass under them
-				 * ;-) */
-				if (!IsTileDepotType(tile, TRANSPORT_RAIL)) {
-					SB(_m[tile].m2, 0, 4, 0);
-					MarkTileDirtyByTile(tile);
-				}
-				break;
-			case MP_STREET:
-				if (!IsTileDepotType(tile, TRANSPORT_ROAD)) {
-					SB(_m[tile].m4, 4, 3, 0);
-					MarkTileDirtyByTile(tile);
-				}
-				break;
-			default:
-				break;
-		}
+#ifndef NO_DEBUG_MESSAGES
+	if (_debug_npf_level < 1 || _networking) return;
+	switch (GetTileType(tile)) {
+		case MP_RAILWAY:
+			/* DEBUG: mark visited tiles by mowing the grass under them ;-) */
+			if (!IsTileDepotType(tile, TRANSPORT_RAIL)) {
+				SetRailGroundType(tile, RAIL_GROUND_BARREN);
+				MarkTileDirtyByTile(tile);
+			}
+			break;
+
+		case MP_STREET:
+			if (!IsTileDepotType(tile, TRANSPORT_ROAD)) {
+				SetRoadside(tile, ROADSIDE_BARREN);
+				MarkTileDirtyByTile(tile);
+			}
+			break;
+
+		default:
+			break;
+	}
 #endif
 }
 
@@ -257,18 +265,19 @@ static int32 NPFRoadPathCost(AyStar* as, AyStarNode* current, OpenListNode* pare
 	/* Determine base length */
 	switch (GetTileType(tile)) {
 		case MP_TUNNELBRIDGE:
-			if (GB(_m[tile].m5, 4, 4) == 0) {
+			if (IsTunnel(tile)) {
 				cost = NPFTunnelCost(current);
-				break;
+			} else {
+				cost = NPF_TILE_LENGTH;
 			}
-			cost = NPF_TILE_LENGTH;
 			break;
+
 		case MP_STREET:
 			cost = NPF_TILE_LENGTH;
 			/* Increase the cost for level crossings */
-			if (IsLevelCrossing(tile))
-				cost += _patches.npf_crossing_penalty;
+			if (IsLevelCrossing(tile)) cost += _patches.npf_crossing_penalty;
 			break;
+
 		default:
 			break;
 	}
@@ -295,34 +304,37 @@ static int32 NPFRailPathCost(AyStar* as, AyStarNode* current, OpenListNode* pare
 	TileIndex tile = current->tile;
 	Trackdir trackdir = (Trackdir)current->direction;
 	int32 cost = 0;
-	/* HACK: We create a OpenListNode manualy, so we can call EndNodeCheck */
+	/* HACK: We create a OpenListNode manually, so we can call EndNodeCheck */
 	OpenListNode new_node;
 
 	/* Determine base length */
 	switch (GetTileType(tile)) {
 		case MP_TUNNELBRIDGE:
-			if (GB(_m[tile].m5, 4, 4) == 0) {
+			if (IsTunnel(tile)) {
 				cost = NPFTunnelCost(current);
 				break;
 			}
 			/* Fall through if above if is false, it is a bridge
 			 * then. We treat that as ordinary rail */
+
 		case MP_RAILWAY:
 			cost = _trackdir_length[trackdir]; /* Should be different for diagonal tracks */
 			break;
+
 		case MP_STREET: /* Railway crossing */
 			cost = NPF_TILE_LENGTH;
 			break;
+
 		case MP_STATION:
-			/* We give a station tile a penalty. Logically we would only
-					* want to give station tiles that are not our destination
-					* this penalty. This would discourage trains to drive through
-					* busy stations. But, we can just give any station tile a
-					* penalty, because every possible route will get this penalty
-					* exactly once, on its end tile (if it's a station) and it
-			* will therefore not make a difference. */
+			/* We give a station tile a penalty. Logically we would only want to give
+			 * station tiles that are not our destination this penalty. This would
+			 * discourage trains to drive through busy stations. But, we can just
+			 * give any station tile a penalty, because every possible route will get
+			 * this penalty exactly once, on its end tile (if it's a station) and it
+			 * will therefore not make a difference. */
 			cost = NPF_TILE_LENGTH + _patches.npf_rail_station_penalty;
 			break;
+
 		default:
 			break;
 	}
@@ -332,19 +344,20 @@ static int32 NPFRailPathCost(AyStar* as, AyStarNode* current, OpenListNode* pare
 	/* Check for signals */
 	if (IsTileType(tile, MP_RAILWAY) && HasSignalOnTrackdir(tile, trackdir)) {
 		/* Ordinary track with signals */
-		if (GetSignalState(tile, trackdir) == SIGNAL_STATE_RED) {
+		if (GetSignalStateByTrackdir(tile, trackdir) == SIGNAL_STATE_RED) {
 			/* Signal facing us is red */
 			if (!NPFGetFlag(current, NPF_FLAG_SEEN_SIGNAL)) {
 				/* Penalize the first signal we
 				 * encounter, if it is red */
 
 				/* Is this a presignal exit or combo? */
-				SignalType sigtype = GetSignalType(tile, TrackdirToTrack(trackdir));
-				if (sigtype == SIGTYPE_EXIT || sigtype == SIGTYPE_COMBO)
+				SignalType sigtype = GetSignalType(tile);
+				if (sigtype == SIGTYPE_EXIT || sigtype == SIGTYPE_COMBO) {
 					/* Penalise exit and combo signals differently (heavier) */
 					cost += _patches.npf_rail_firstred_exit_penalty;
-				else
+				} else {
 					cost += _patches.npf_rail_firstred_penalty;
+				}
 			}
 			/* Record the state of this signal */
 			NPFSetFlag(current, NPF_FLAG_LAST_SIGNAL_RED, true);
@@ -372,14 +385,12 @@ static int32 NPFRailPathCost(AyStar* as, AyStarNode* current, OpenListNode* pare
 	//TODO, with realistic acceleration, also the amount of straight track between
 	//      curves should be taken into account, as this affects the speed limit.
 
-
 	/* Check for reverse in depot */
 	if (IsTileDepotType(tile, TRANSPORT_RAIL) && as->EndNodeCheck(as, &new_node) != AYSTAR_FOUND_END_NODE) {
 		/* Penalise any depot tile that is not the last tile in the path. This
 		 * _should_ penalise every occurence of reversing in a depot (and only
 		 * that) */
 		cost += _patches.npf_rail_depot_reverse_penalty;
-
 	}
 
 	/* Check for occupied track */
@@ -393,14 +404,10 @@ static int32 NPFRailPathCost(AyStar* as, AyStarNode* current, OpenListNode* pare
 /* Will find any depot */
 static int32 NPFFindDepot(AyStar* as, OpenListNode *current)
 {
-	TileIndex tile = current->path.node.tile;
-
 	/* It's not worth caching the result with NPF_FLAG_IS_TARGET here as below,
 	 * since checking the cache not that much faster than the actual check */
-	if (IsTileDepotType(tile, as->user_data[NPF_TYPE]))
-		return AYSTAR_FOUND_END_NODE;
-	else
-		return AYSTAR_DONE;
+	return IsTileDepotType(current->path.node.tile, as->user_data[NPF_TYPE]) ?
+		AYSTAR_FOUND_END_NODE : AYSTAR_DONE;
 }
 
 /* Will find a station identified using the NPFFindStationOrTileData */
@@ -413,8 +420,8 @@ static int32 NPFFindStationOrTile(AyStar* as, OpenListNode *current)
 	/* If GetNeighbours said we could get here, we assume the station type
 	 * is correct */
 	if (
-		(fstd->station_index == -1 && tile == fstd->dest_coords) || /* We've found the tile, or */
-		(IsTileType(tile, MP_STATION) && _m[tile].m2 == fstd->station_index) /* the station */
+		(fstd->station_index == INVALID_STATION && tile == fstd->dest_coords) || /* We've found the tile, or */
+		(IsTileType(tile, MP_STATION) && GetStationIndex(tile) == fstd->station_index) /* the station */
 	) {
 		return AYSTAR_FOUND_END_NODE;
 	} else {
@@ -446,42 +453,40 @@ static void NPFSaveTargetData(AyStar* as, OpenListNode* current)
  */
 static bool VehicleMayEnterTile(Owner owner, TileIndex tile, DiagDirection enterdir)
 {
-	if (
-		IsTileType(tile, MP_RAILWAY) /* Rail tile (also rail depot) */
-		|| IsTrainStationTile(tile) /* Rail station tile */
-		|| IsTileDepotType(tile, TRANSPORT_ROAD) /* Road depot tile */
-		|| IsRoadStationTile(tile) /* Road station tile */
-		|| IsTileDepotType(tile, TRANSPORT_WATER) /* Water depot tile */
-		)
+	if (IsTileType(tile, MP_RAILWAY) ||           /* Rail tile (also rail depot) */
+			IsRailwayStationTile(tile) ||               /* Rail station tile */
+			IsTileDepotType(tile, TRANSPORT_ROAD) ||  /* Road depot tile */
+			IsRoadStopTile(tile) ||                /* Road station tile */
+			IsTileDepotType(tile, TRANSPORT_WATER)) { /* Water depot tile */
 		return IsTileOwner(tile, owner); /* You need to own these tiles entirely to use them */
+	}
 
 	switch (GetTileType(tile)) {
 		case MP_STREET:
 			/* rail-road crossing : are we looking at the railway part? */
-			if (IsLevelCrossing(tile) && GetCrossingTransportType(tile, TrackdirToTrack(DiagdirToDiagTrackdir(enterdir))) == TRANSPORT_RAIL)
+			if (IsLevelCrossing(tile) &&
+					DiagDirToAxis(enterdir) != GetCrossingRoadAxis(tile)) {
 				return IsTileOwner(tile, owner); /* Railway needs owner check, while the street is public */
-			break;
-		case MP_TUNNELBRIDGE:
-#if 0
-/* OPTIMISATION: If we are on the middle of a bridge, we will not do the cpu
- * intensive owner check, instead we will just assume that if the vehicle
- * managed to get on the bridge, it is probably allowed to :-)
- */
-			if ((_m[tile].m5 & 0xC6) == 0xC0 && GB(_m[tile].m5, 0, 1) == (enterdir & 0x1)) {
-				/* on the middle part of a railway bridge: find bridge ending */
-				while (IsTileType(tile, MP_TUNNELBRIDGE) && !((_m[tile].m5 & 0xC6) == 0x80)) {
-					tile += TileOffsByDir(GB(_m[tile].m5, 0, 1));
-				}
 			}
-			/* if we were on a railway middle part, we are now at a railway bridge ending */
-#endif
-			if (
-				(_m[tile].m5 & 0xFC) == 0 /* railway tunnel */
-				|| (_m[tile].m5 & 0xC6) == 0x80 /* railway bridge ending */
-				|| ((_m[tile].m5 & 0xF8) == 0xE0 && GB(_m[tile].m5, 0, 1) != (enterdir & 0x1)) /* railway under bridge */
-				)
-				return IsTileOwner(tile, owner);
 			break;
+
+		case MP_TUNNELBRIDGE:
+			if ((IsTunnel(tile) && GetTunnelTransportType(tile) == TRANSPORT_RAIL) ||
+					(IsBridge(tile) && (
+						(
+							IsBridgeRamp(tile) &&
+							GetBridgeTransportType(tile) == TRANSPORT_RAIL
+						) || (
+							IsBridgeMiddle(tile) &&
+							IsTransportUnderBridge(tile) &&
+							GetTransportTypeUnderBridge(tile) == TRANSPORT_RAIL &&
+							GetBridgeAxis(tile) != DiagDirToAxis(enterdir)
+						)
+					))) {
+				return IsTileOwner(tile, owner);
+			}
+			break;
+
 		default:
 			break;
 	}
@@ -489,6 +494,23 @@ static bool VehicleMayEnterTile(Owner owner, TileIndex tile, DiagDirection enter
 	return true; /* no need to check */
 }
 
+
+/**
+ * Returns the direction the exit of the depot on the given tile is facing.
+ */
+static DiagDirection GetDepotDirection(TileIndex tile, TransportType type)
+{
+	assert(IsTileDepotType(tile, type));
+
+	switch (type) {
+		case TRANSPORT_RAIL:  return GetRailDepotDirection(tile);
+		case TRANSPORT_ROAD:  return GetRoadDepotDirection(tile);
+		case TRANSPORT_WATER: return GetShipDepotDirection(tile);
+		default: return INVALID_DIAGDIR; /* Not reached */
+	}
+}
+
+
 /* Will just follow the results of GetTileTrackStatus concerning where we can
  * go and where not. Uses AyStar.user_data[NPF_TYPE] as the transport type and
  * an argument to GetTileTrackStatus. Will skip tunnels, meaning that the
@@ -500,7 +522,6 @@ static void NPFFollowTrack(AyStar* aystar, OpenListNode* current)
 	Trackdir src_trackdir = (Trackdir)current->path.node.direction;
 	TileIndex src_tile = current->path.node.tile;
 	DiagDirection src_exitdir = TrackdirToExitdir(src_trackdir);
-	FindLengthOfTunnelResult flotr;
 	TileIndex dst_tile = INVALID_TILE;
 	int i;
 	TrackdirBits trackdirbits, ts;
@@ -510,28 +531,26 @@ static void NPFFollowTrack(AyStar* aystar, OpenListNode* current)
 	DEBUG(npf, 4)("Expanding: (%d, %d, %d) [%d]", TileX(src_tile), TileY(src_tile), src_trackdir, src_tile);
 
 	/* Find dest tile */
-	if (IsTileType(src_tile, MP_TUNNELBRIDGE) && GB(_m[src_tile].m5, 4, 4) == 0 &&
-			(DiagDirection)GB(_m[src_tile].m5, 0, 2) == src_exitdir) {
+	if (IsTunnelTile(src_tile) && GetTunnelDirection(src_tile) == src_exitdir) {
 		/* This is a tunnel. We know this tunnel is our type,
 		 * otherwise we wouldn't have got here. It is also facing us,
 		 * so we should skip it's body */
-		flotr = FindLengthOfTunnel(src_tile, src_exitdir);
-		dst_tile = flotr.tile;
-	} else if (type != TRANSPORT_WATER && (IsRoadStationTile(src_tile) || IsTileDepotType(src_tile, type))) {
+		dst_tile = GetOtherTunnelEnd(src_tile);
+	} else if (type != TRANSPORT_WATER && (IsRoadStopTile(src_tile) || IsTileDepotType(src_tile, type))) {
 		/* This is a road station or a train or road depot. We can enter and exit
 		 * those from one side only. Trackdirs don't support that (yet), so we'll
 		 * do this here. */
 
 		DiagDirection exitdir;
 		/* Find out the exit direction first */
-		if (IsRoadStationTile(src_tile)) {
-			exitdir = GetRoadStationDir(src_tile);
-		} else { /* Train or road depot. Direction is stored the same for both, in map5 */
+		if (IsRoadStopTile(src_tile)) {
+			exitdir = GetRoadStopDir(src_tile);
+		} else { /* Train or road depot */
 			exitdir = GetDepotDirection(src_tile, type);
 		}
 
 		/* Let's see if were headed the right way into the depot */
-		if (src_trackdir == DiagdirToDiagTrackdir(ReverseDiagdir(exitdir))) {
+		if (src_trackdir == DiagdirToDiagTrackdir(ReverseDiagDir(exitdir))) {
 			/* We are headed inwards. We cannot go through the back of the depot.
 			 * For rail, we can now reverse. Reversing for road vehicles is never
 			 * useful, since you cannot take paths you couldn't take before
@@ -544,16 +563,16 @@ static void NPFFollowTrack(AyStar* aystar, OpenListNode* current)
 				 * outside of the depot. But, because the code layout is cleaner this
 				 * way, we will just pretend we are reversed already */
 				src_trackdir = ReverseTrackdir(src_trackdir);
-				dst_tile = AddTileIndexDiffCWrap(src_tile, TileIndexDiffCByDir(exitdir));
+				dst_tile = AddTileIndexDiffCWrap(src_tile, TileIndexDiffCByDiagDir(exitdir));
 			} else {
 				dst_tile = INVALID_TILE; /* Road vehicle heading inwards: dead end */
 			}
 		} else {
-			dst_tile = AddTileIndexDiffCWrap(src_tile, TileIndexDiffCByDir(exitdir));
+			dst_tile = AddTileIndexDiffCWrap(src_tile, TileIndexDiffCByDiagDir(exitdir));
 		}
 	} else {
 		/* This a normal tile, a bridge, a tunnel exit, etc. */
-		dst_tile = AddTileIndexDiffCWrap(src_tile, TileIndexDiffCByDir(TrackdirToExitdir(src_trackdir)));
+		dst_tile = AddTileIndexDiffCWrap(src_tile, TileIndexDiffCByDiagDir(TrackdirToExitdir(src_trackdir)));
 	}
 	if (dst_tile == INVALID_TILE) {
 		/* We reached the border of the map */
@@ -564,15 +583,14 @@ static void NPFFollowTrack(AyStar* aystar, OpenListNode* current)
 	/* I can't enter a tunnel entry/exit tile from a tile above the tunnel. Note
 	 * that I can enter the tunnel from a tile below the tunnel entrance. This
 	 * solves the problem of vehicles wanting to drive off a tunnel entrance */
-	if (IsTileType(dst_tile, MP_TUNNELBRIDGE) && GB(_m[dst_tile].m5, 4, 4) == 0 &&
-			GetTileZ(dst_tile) < GetTileZ(src_tile)) {
+	if (IsTunnelTile(dst_tile) && GetTileZ(dst_tile) < GetTileZ(src_tile)) {
 		return;
 	}
 
 	/* check correct rail type (mono, maglev, etc) */
 	if (type == TRANSPORT_RAIL) {
 		RailType dst_type = GetTileRailType(dst_tile, src_trackdir);
-		if (!IsCompatibleRail(aystar->user_data[NPF_RAILTYPE], dst_type))
+		if (!HASBIT(aystar->user_data[NPF_RAILTYPES], dst_type))
 			return;
 	}
 
@@ -582,30 +600,31 @@ static void NPFFollowTrack(AyStar* aystar, OpenListNode* current)
 	}
 
 	/* Determine available tracks */
-	if (type != TRANSPORT_WATER && (IsRoadStationTile(dst_tile) || IsTileDepotType(dst_tile, type))){
+	if (type != TRANSPORT_WATER && (IsRoadStopTile(dst_tile) || IsTileDepotType(dst_tile, type))){
 		/* Road stations and road and train depots return 0 on GTTS, so we have to do this by hand... */
 		DiagDirection exitdir;
-		if (IsRoadStationTile(dst_tile))
-			exitdir = GetRoadStationDir(dst_tile);
-		else /* Road or train depot */
+		if (IsRoadStopTile(dst_tile)) {
+			exitdir = GetRoadStopDir(dst_tile);
+		} else { /* Road or train depot */
 			exitdir = GetDepotDirection(dst_tile, type);
+		}
 		/* Find the trackdirs that are available for a depot or station with this
 		 * orientation. They are only "inwards", since we are reaching this tile
 		 * from some other tile. This prevents vehicles driving into depots from
 		 * the back */
-		ts = TrackdirToTrackdirBits(DiagdirToDiagTrackdir(ReverseDiagdir(exitdir)));
+		ts = TrackdirToTrackdirBits(DiagdirToDiagTrackdir(ReverseDiagDir(exitdir)));
 	} else {
 		ts = GetTileTrackStatus(dst_tile, type);
 	}
 	trackdirbits = ts & TRACKDIR_BIT_MASK; /* Filter out signal status and the unused bits */
 
-	DEBUG(npf, 4)("Next node: (%d, %d) [%d], possible trackdirs: %#x", TileX(dst_tile), TileY(dst_tile), dst_tile, trackdirbits);
+	DEBUG(npf, 4)("Next node: (%d, %d) [%d], possible trackdirs: 0x%X", TileX(dst_tile), TileY(dst_tile), dst_tile, trackdirbits);
 	/* Select only trackdirs we can reach from our current trackdir */
 	trackdirbits &= TrackdirReachesTrackdirs(src_trackdir);
 	if (_patches.forbid_90_deg && (type == TRANSPORT_RAIL || type == TRANSPORT_WATER)) /* Filter out trackdirs that would make 90 deg turns for trains */
 		trackdirbits &= ~TrackdirCrossesTrackdirs(src_trackdir);
 
-	DEBUG(npf,6)("After filtering: (%d, %d), possible trackdirs: %#x", TileX(dst_tile), TileY(dst_tile), trackdirbits);
+	DEBUG(npf,6)("After filtering: (%d, %d), possible trackdirs: 0x%X", TileX(dst_tile), TileY(dst_tile), trackdirbits);
 
 	i = 0;
 	/* Enumerate possible track */
@@ -613,10 +632,10 @@ static void NPFFollowTrack(AyStar* aystar, OpenListNode* current)
 		Trackdir dst_trackdir;
 		dst_trackdir =  FindFirstBit2x64(trackdirbits);
 		trackdirbits = KillFirstBit2x64(trackdirbits);
-		DEBUG(npf, 5)("Expanded into trackdir: %d, remaining trackdirs: %#x", dst_trackdir, trackdirbits);
+		DEBUG(npf, 5)("Expanded into trackdir: %d, remaining trackdirs: 0x%X", dst_trackdir, trackdirbits);
 
 		/* Check for oneway signal against us */
-		if (IsTileType(dst_tile, MP_RAILWAY) && GetRailTileType(dst_tile) == RAIL_TYPE_SIGNALS) {
+		if (IsTileType(dst_tile, MP_RAILWAY) && GetRailTileType(dst_tile) == RAIL_TILE_SIGNALS) {
 			if (HasSignalOnTrackdir(dst_tile, ReverseTrackdir(dst_trackdir)) && !HasSignalOnTrackdir(dst_tile, dst_trackdir))
 				// if one way signal not pointing towards us, stop going in this direction.
 				break;
@@ -645,7 +664,7 @@ static void NPFFollowTrack(AyStar* aystar, OpenListNode* current)
  * multiple targets that are spread around, we should perform a breadth first
  * search by specifiying CalcZero as our heuristic.
  */
-static NPFFoundTargetData NPFRouteInternal(AyStarNode* start1, AyStarNode* start2, NPFFindStationOrTileData* target, AyStar_EndNodeCheck target_proc, AyStar_CalculateH heuristic_proc, TransportType type, Owner owner, RailType railtype, uint reverse_penalty)
+static NPFFoundTargetData NPFRouteInternal(AyStarNode* start1, AyStarNode* start2, NPFFindStationOrTileData* target, AyStar_EndNodeCheck target_proc, AyStar_CalculateH heuristic_proc, TransportType type, Owner owner, RailTypeMask railtypes, uint reverse_penalty)
 {
 	int r;
 	NPFFoundTargetData result;
@@ -655,14 +674,12 @@ static NPFFoundTargetData NPFRouteInternal(AyStarNode* start1, AyStarNode* start
 	_npf_aystar.EndNodeCheck = target_proc;
 	_npf_aystar.FoundEndNode = NPFSaveTargetData;
 	_npf_aystar.GetNeighbours = NPFFollowTrack;
-	if (type == TRANSPORT_RAIL)
-		_npf_aystar.CalculateG = NPFRailPathCost;
-	else if (type == TRANSPORT_ROAD)
-		_npf_aystar.CalculateG = NPFRoadPathCost;
-	else if (type == TRANSPORT_WATER)
-		_npf_aystar.CalculateG = NPFWaterPathCost;
-	else
-		assert(0);
+	switch (type) {
+		default: NOT_REACHED();
+		case TRANSPORT_RAIL:  _npf_aystar.CalculateG = NPFRailPathCost;  break;
+		case TRANSPORT_ROAD:  _npf_aystar.CalculateG = NPFRoadPathCost;  break;
+		case TRANSPORT_WATER: _npf_aystar.CalculateG = NPFWaterPathCost; break;
+	}
 
 	/* Initialize Start Node(s) */
 	start1->user_data[NPF_TRACKDIR_CHOICE] = INVALID_TRACKDIR;
@@ -687,25 +704,25 @@ static NPFFoundTargetData NPFRouteInternal(AyStarNode* start1, AyStarNode* start
 	/* Initialize user_data */
 	_npf_aystar.user_data[NPF_TYPE] = type;
 	_npf_aystar.user_data[NPF_OWNER] = owner;
-	_npf_aystar.user_data[NPF_RAILTYPE] = railtype;
+	_npf_aystar.user_data[NPF_RAILTYPES] = railtypes;
 
 	/* GO! */
 	r = AyStarMain_Main(&_npf_aystar);
 	assert(r != AYSTAR_STILL_BUSY);
 
 	if (result.best_bird_dist != 0) {
-		if (target) {
-			DEBUG(misc, 1) ("NPF: Could not find route to 0x%x from 0x%x.", target->dest_coords, start1->tile);
+		if (target != NULL) {
+			DEBUG(npf, 1) ("Could not find route to tile 0x%X from 0x%X.", target->dest_coords, start1->tile);
 		} else {
 			/* Assumption: target == NULL, so we are looking for a depot */
-			DEBUG(misc, 1) ("NPF: Could not find route to a depot from 0x%x.", start1->tile);
+			DEBUG(npf, 1) ("Could not find route to a depot from tile 0x%X.", start1->tile);
 		}
 
 	}
 	return result;
 }
 
-NPFFoundTargetData NPFRouteToStationOrTileTwoWay(TileIndex tile1, Trackdir trackdir1, TileIndex tile2, Trackdir trackdir2, NPFFindStationOrTileData* target, TransportType type, Owner owner, RailType railtype)
+NPFFoundTargetData NPFRouteToStationOrTileTwoWay(TileIndex tile1, Trackdir trackdir1, TileIndex tile2, Trackdir trackdir2, NPFFindStationOrTileData* target, TransportType type, Owner owner, RailTypeMask railtypes)
 {
 	AyStarNode start1;
 	AyStarNode start2;
@@ -719,15 +736,15 @@ NPFFoundTargetData NPFRouteToStationOrTileTwoWay(TileIndex tile1, Trackdir track
 	start2.direction = trackdir2;
 	start2.user_data[NPF_TRACKDIR_CHOICE] = INVALID_TRACKDIR;
 
-	return NPFRouteInternal(&start1, (IsValidTile(tile2) ? &start2 : NULL), target, NPFFindStationOrTile, NPFCalcStationOrTileHeuristic, type, owner, railtype, 0);
+	return NPFRouteInternal(&start1, (IsValidTile(tile2) ? &start2 : NULL), target, NPFFindStationOrTile, NPFCalcStationOrTileHeuristic, type, owner, railtypes, 0);
 }
 
-NPFFoundTargetData NPFRouteToStationOrTile(TileIndex tile, Trackdir trackdir, NPFFindStationOrTileData* target, TransportType type, Owner owner, RailType railtype)
+NPFFoundTargetData NPFRouteToStationOrTile(TileIndex tile, Trackdir trackdir, NPFFindStationOrTileData* target, TransportType type, Owner owner, RailTypeMask railtypes)
 {
-	return NPFRouteToStationOrTileTwoWay(tile, trackdir, INVALID_TILE, 0, target, type, owner, railtype);
+	return NPFRouteToStationOrTileTwoWay(tile, trackdir, INVALID_TILE, 0, target, type, owner, railtypes);
 }
 
-NPFFoundTargetData NPFRouteToDepotBreadthFirstTwoWay(TileIndex tile1, Trackdir trackdir1, TileIndex tile2, Trackdir trackdir2, TransportType type, Owner owner, RailType railtype, uint reverse_penalty)
+NPFFoundTargetData NPFRouteToDepotBreadthFirstTwoWay(TileIndex tile1, Trackdir trackdir1, TileIndex tile2, Trackdir trackdir2, TransportType type, Owner owner, RailTypeMask railtypes, uint reverse_penalty)
 {
 	AyStarNode start1;
 	AyStarNode start2;
@@ -743,15 +760,15 @@ NPFFoundTargetData NPFRouteToDepotBreadthFirstTwoWay(TileIndex tile1, Trackdir t
 
 	/* perform a breadth first search. Target is NULL,
 	 * since we are just looking for any depot...*/
-	return NPFRouteInternal(&start1, (IsValidTile(tile2) ? &start2 : NULL), NULL, NPFFindDepot, NPFCalcZero, type, owner, railtype, reverse_penalty);
+	return NPFRouteInternal(&start1, (IsValidTile(tile2) ? &start2 : NULL), NULL, NPFFindDepot, NPFCalcZero, type, owner, railtypes, reverse_penalty);
 }
 
-NPFFoundTargetData NPFRouteToDepotBreadthFirst(TileIndex tile, Trackdir trackdir, TransportType type, Owner owner, RailType railtype)
+NPFFoundTargetData NPFRouteToDepotBreadthFirst(TileIndex tile, Trackdir trackdir, TransportType type, Owner owner, RailTypeMask railtypes)
 {
-	return NPFRouteToDepotBreadthFirstTwoWay(tile, trackdir, INVALID_TILE, 0, type, owner, railtype, 0);
+	return NPFRouteToDepotBreadthFirstTwoWay(tile, trackdir, INVALID_TILE, 0, type, owner, railtypes, 0);
 }
 
-NPFFoundTargetData NPFRouteToDepotTrialError(TileIndex tile, Trackdir trackdir, TransportType type, Owner owner, RailType railtype)
+NPFFoundTargetData NPFRouteToDepotTrialError(TileIndex tile, Trackdir trackdir, TransportType type, Owner owner, RailTypeMask railtypes)
 {
 	/* Okay, what we're gonna do. First, we look at all depots, calculate
 	 * the manhatten distance to get to each depot. We then sort them by
@@ -763,7 +780,7 @@ NPFFoundTargetData NPFRouteToDepotTrialError(TileIndex tile, Trackdir trackdir,
 	 */
 	Queue depots;
 	int r;
-	NPFFoundTargetData best_result;
+	NPFFoundTargetData best_result = {(uint)-1, (uint)-1, INVALID_TRACKDIR, {INVALID_TILE, 0, {0, 0}}};
 	NPFFoundTargetData result;
 	NPFFindStationOrTileData target;
 	AyStarNode start;
@@ -775,7 +792,7 @@ NPFFoundTargetData NPFRouteToDepotTrialError(TileIndex tile, Trackdir trackdir,
 	FOR_ALL_DEPOTS(depot) {
 		/* Check if this is really a valid depot, it is of the needed type and
 		 * owner */
-		if (IsValidDepot(depot) && IsTileDepotType(depot->xy, type) && IsTileOwner(depot->xy, owner))
+		if (IsTileDepotType(depot->xy, type) && IsTileOwner(depot->xy, owner))
 			/* If so, let's add it to the queue, sorted by distance */
 			depots.push(&depots, depot, DistanceManhattan(tile, depot->xy));
 	}
@@ -787,17 +804,15 @@ NPFFoundTargetData NPFRouteToDepotTrialError(TileIndex tile, Trackdir trackdir,
 	_npf_aystar.EndNodeCheck = NPFFindStationOrTile;
 	_npf_aystar.FoundEndNode = NPFSaveTargetData;
 	_npf_aystar.GetNeighbours = NPFFollowTrack;
-	if (type == TRANSPORT_RAIL)
-		_npf_aystar.CalculateG = NPFRailPathCost;
-	else if (type == TRANSPORT_ROAD)
-		_npf_aystar.CalculateG = NPFRoadPathCost;
-	else if (type == TRANSPORT_WATER)
-		_npf_aystar.CalculateG = NPFWaterPathCost;
-	else
-		assert(0);
+	switch (type) {
+		default: NOT_REACHED();
+		case TRANSPORT_RAIL:  _npf_aystar.CalculateG = NPFRailPathCost;  break;
+		case TRANSPORT_ROAD:  _npf_aystar.CalculateG = NPFRoadPathCost;  break;
+		case TRANSPORT_WATER: _npf_aystar.CalculateG = NPFWaterPathCost; break;
+	}
 
 	/* Initialize target */
-	target.station_index = -1; /* We will initialize dest_coords inside the loop below */
+	target.station_index = INVALID_STATION; /* We will initialize dest_coords inside the loop below */
 	_npf_aystar.user_target = &target;
 
 	/* Initialize user_data */
@@ -846,14 +861,20 @@ NPFFoundTargetData NPFRouteToDepotTrialError(TileIndex tile, Trackdir trackdir,
 			best_result = result;
 	}
 	if (result.best_bird_dist != 0) {
-		DEBUG(misc, 1) ("NPF: Could not find route to any depot from 0x%x.", tile);
+		DEBUG(npf, 1) ("Could not find route to any depot from tile 0x%X.", tile);
 	}
 	return best_result;
 }
 
 void InitializeNPF(void)
 {
-	init_AyStar(&_npf_aystar, NPFHash, NPF_HASH_SIZE);
+	static bool first_init = true;
+	if (first_init) {
+		first_init = false;
+		init_AyStar(&_npf_aystar, NPFHash, NPF_HASH_SIZE);
+	} else {
+		AyStarMain_Clear(&_npf_aystar);
+	}
 	_npf_aystar.loops_per_tick = 0;
 	_npf_aystar.max_path_cost = 0;
 	//_npf_aystar.max_search_nodes = 0;
@@ -870,12 +891,12 @@ void NPFFillWithOrderData(NPFFindStationOrTileData* fstd, Vehicle* v)
 	 * dest_tile, not just any stop of that station.
 	 * So only for train orders to stations we fill fstd->station_index, for all
 	 * others only dest_coords */
-	if ((v->current_order.type) == OT_GOTO_STATION && v->type == VEH_Train) {
-		fstd->station_index = v->current_order.station;
+	if (v->current_order.type == OT_GOTO_STATION && v->type == VEH_Train) {
+		fstd->station_index = v->current_order.dest;
 		/* Let's take the closest tile of the station as our target for trains */
-		fstd->dest_coords = CalcClosestStationTile(v->current_order.station, v->tile);
+		fstd->dest_coords = CalcClosestStationTile(v->current_order.dest, v->tile);
 	} else {
 		fstd->dest_coords = v->dest_tile;
-		fstd->station_index = -1;
+		fstd->station_index = INVALID_STATION;
 	}
 }
