@@ -1,7 +1,8 @@
-/* $Id: oldloader.c 3472 2006-01-29 18:57:26Z Darkvater $ */
+/* $Id: oldloader.c 10735 2007-07-30 10:24:04Z rubidium $ */
 
 #include "stdafx.h"
 #include "openttd.h"
+#include "station_map.h"
 #include "table/strings.h"
 #include "functions.h"
 #include "map.h"
@@ -17,6 +18,8 @@
 #include "depot.h"
 #include "network.h"
 #include "ai/ai.h"
+#include "date.h"
+#include "bridge_map.h"
 
 enum {
 	HEADER_SIZE = 49,
@@ -41,38 +44,30 @@ typedef struct LoadgameState {
 	bool failed;
 } LoadgameState;
 
-typedef bool OldChunkProc(LoadgameState *ls, int num);
-
-typedef struct OldChunks {
-	uint32 type;         //! Type of field
-	uint32 amount;       //! Amount of fields
-
-	void *ptr;           //! Pointer where to save the data (may only be set if offset is 0)
-	uint offset;         //! Offset from basepointer (may only be set if ptr is NULL)
-	OldChunkProc *proc;  //! Pointer to function that is called with OC_CHUNK
-} OldChunks;
-
 /* OldChunk-Type */
-enum {
-	OC_END       = 0,
-	OC_NULL      = 1 << 0,
-	OC_CHUNK     = 1 << 1,
-	OC_ASSERT    = 1 << 2,
+typedef enum OldChunkTypes {
+	OC_SIMPLE    = 0,
+	OC_NULL      = 1,
+	OC_CHUNK     = 2,
+	OC_ASSERT    = 3,
+	/* 8 bytes allocated (256 max) */
 
-	OC_VAR_I8    = 1 << 9,
-	OC_VAR_U8    = 1 << 10,
-	OC_VAR_I16   = 1 << 11,
-	OC_VAR_U16   = 1 << 12,
-	OC_VAR_I32   = 1 << 13,
-	OC_VAR_U32   = 1 << 14,
-	OC_VAR_I64   = 1 << 15,
+	OC_VAR_I8    = 1 << 8,
+	OC_VAR_U8    = 2 << 8,
+	OC_VAR_I16   = 3 << 8,
+	OC_VAR_U16   = 4 << 8,
+	OC_VAR_I32   = 5 << 8,
+	OC_VAR_U32   = 6 << 8,
+	OC_VAR_I64   = 7 << 8,
+	/* 8 bytes allocated (256 max) */
 
-	OC_FILE_I8   = 1 << 17,
-	OC_FILE_U8   = 1 << 18,
-	OC_FILE_I16  = 1 << 19,
-	OC_FILE_U16  = 1 << 20,
-	OC_FILE_I32  = 1 << 21,
-	OC_FILE_U32  = 1 << 22,
+	OC_FILE_I8   = 1 << 16,
+	OC_FILE_U8   = 2 << 16,
+	OC_FILE_I16  = 3 << 16,
+	OC_FILE_U16  = 4 << 16,
+	OC_FILE_I32  = 5 << 16,
+	OC_FILE_U32  = 6 << 16,
+	/* 8 bytes allocated (256 max) */
 
 	OC_INT8      = OC_VAR_I8   | OC_FILE_I8,
 	OC_UINT8     = OC_VAR_U8   | OC_FILE_U8,
@@ -81,14 +76,40 @@ enum {
 	OC_INT32     = OC_VAR_I32  | OC_FILE_I32,
 	OC_UINT32    = OC_VAR_U32  | OC_FILE_U32,
 
-	OC_TILE      = OC_VAR_U32  | OC_FILE_U16
-};
+	OC_TILE      = OC_VAR_U32  | OC_FILE_U16,
+
+	OC_END       = 0 ///< End of the whole chunk, all 32bits set to zero
+} OldChunkType;
+
+typedef bool OldChunkProc(LoadgameState *ls, int num);
+
+typedef struct OldChunks {
+	OldChunkType type;   ///< Type of field
+	uint32 amount;       ///< Amount of fields
+
+	void *ptr;           ///< Pointer where to save the data (may only be set if offset is 0)
+	uint offset;         ///< Offset from basepointer (may only be set if ptr is NULL)
+	OldChunkProc *proc;  ///< Pointer to function that is called with OC_CHUNK
+} OldChunks;
+
 /* If it fails, check lines above.. */
 assert_compile(sizeof(TileIndex) == 4);
 
 static uint32 _bump_assert_value;
 static bool   _read_ttdpatch_flags;
 
+static OldChunkType GetOldChunkType(OldChunkType type)     {return GB(type, 0, 8);}
+static OldChunkType GetOldChunkVarType(OldChunkType type)  {return GB(type, 8, 8) << 8;}
+static OldChunkType GetOldChunkFileType(OldChunkType type) {return GB(type, 16, 8) << 16;}
+
+static inline byte CalcOldVarLen(OldChunkType type)
+{
+	static const byte type_mem_size[] = {0, 1, 1, 2, 2, 4, 4, 8};
+	byte length = GB(type, 8, 8);
+	assert(length != 0 && length < lengthof(type_mem_size));
+	return type_mem_size[length];
+}
+
 /**
  *
  * Reads a byte from a file (do not call yourself, use ReadByte())
@@ -100,7 +121,7 @@ static byte ReadByteFromFile(LoadgameState *ls)
 	and just return a byte per time */
 	if (ls->buffer_cur >= ls->buffer_count) {
 		/* Read some new bytes from the file */
-		int count = fread(ls->buffer, 1, BUFFER_SIZE, ls->file);
+		int count = (int)fread(ls->buffer, 1, BUFFER_SIZE, ls->file);
 
 		/* We tried to read, but there is nothing in the file anymore.. */
 		if (count == 0) {
@@ -149,6 +170,18 @@ static byte ReadByte(LoadgameState *ls)
 	return ls->decoding ? ls->decode_char : ReadByteFromFile(ls);
 }
 
+static inline uint16 ReadUint16(LoadgameState *ls)
+{
+	byte x = ReadByte(ls);
+	return x | ReadByte(ls) << 8;
+}
+
+static inline uint32 ReadUint32(LoadgameState *ls)
+{
+	uint16 x = ReadUint16(ls);
+	return x | ReadUint16(ls) << 16;
+}
+
 /**
  *
  * Loads a chunk from the old savegame
@@ -157,149 +190,66 @@ static byte ReadByte(LoadgameState *ls)
 static bool LoadChunk(LoadgameState *ls, void *base, const OldChunks *chunks)
 {
 	const OldChunks *chunk = chunks;
-	byte *ptr;
 	byte *base_ptr = base;
-	uint i;
 
 	while (chunk->type != OC_END) {
-		ptr = chunk->ptr;
+		byte* ptr = chunk->ptr;
+		uint i;
 
 		for (i = 0; i < chunk->amount; i++) {
-			if (ls->failed)
-				return false;
+			if (ls->failed) return false;
 
 			/* Handle simple types */
-			if ((chunk->type & 0xFF) != 0) {
-				switch (chunk->type & 0xFF) {
-					case OC_NULL:
-						/* Just read the byte and forget about it */
-						ReadByte(ls);
-						break;
+			if (GetOldChunkType(chunk->type) != 0) {
+				switch (GetOldChunkType(chunk->type)) {
+					/* Just read the byte and forget about it */
+					case OC_NULL: ReadByte(ls); break;
 
 					case OC_CHUNK:
 						/* Call function, with 'i' as parameter to tell which item we
-						are going to read */
-						if (!chunk->proc(ls, i))
-							return false;
+						 * are going to read */
+						if (!chunk->proc(ls, i)) return false;
 						break;
 
 					case OC_ASSERT:
-						DEBUG(oldloader, 4)("[OldLoader] Assert point: %x / %x", ls->total_read, chunk->offset + _bump_assert_value);
-						if (ls->total_read != chunk->offset + _bump_assert_value)
-							ls->failed = true;
-
-						break;
-
+						DEBUG(oldloader, 4)("[OldLoader] Assert point: 0x%X / 0x%X", ls->total_read, chunk->offset + _bump_assert_value);
+						if (ls->total_read != chunk->offset + _bump_assert_value) ls->failed = true;
+					default: break;
 				}
 			} else {
 				uint32 res = 0;
 
-				/* Reading from the file: bit 16 to 23 have the FILE */
-				switch (GB(chunk->type, 16, 8) << 16) {
-					case OC_FILE_I8:
-						res = ReadByte(ls);
-						res = (int8)res;
-						break;
-
-					case OC_FILE_U8:
-						res = ReadByte(ls);
-						break;
-
-					case OC_FILE_U16:
-						res =  ReadByte(ls);
-						res += ReadByte(ls) << 8;
-						break;
-
-					case OC_FILE_I16:
-						res =  ReadByte(ls);
-						res += ReadByte(ls) << 8;
-						res = (int16)res;
-						break;
-
-					case OC_FILE_U32:
-						res =  ReadByte(ls);
-						res += ReadByte(ls) << 8;
-						res += ReadByte(ls) << 16;
-						res += ReadByte(ls) << 24;
-						break;
-
-					case OC_FILE_I32:
-						res =  ReadByte(ls);
-						res += ReadByte(ls) << 8;
-						res += ReadByte(ls) << 16;
-						res += ReadByte(ls) << 24;
-						res = (int32)res;
-						break;
+				/* Reading from the file: bits 16 to 23 have the FILE type */
+				switch (GetOldChunkFileType(chunk->type)) {
+					case OC_FILE_I8:  res = (int8)ReadByte(ls); break;
+					case OC_FILE_U8:  res = ReadByte(ls); break;
+					case OC_FILE_I16: res = (int16)ReadUint16(ls); break;
+					case OC_FILE_U16: res = ReadUint16(ls); break;
+					case OC_FILE_I32: res = (int32)ReadUint32(ls); break;
+					case OC_FILE_U32: res = ReadUint32(ls); break;
+					default: NOT_REACHED();
 				}
 
 				/* Sanity check */
 				assert(base_ptr != NULL || chunk->ptr != NULL);
 
-				/* Writing to the var: bit 8 till 15 have the VAR */
-				switch (GB(chunk->type, 8, 8) << 8) {
-					case OC_VAR_I8:
-						/* Write the data */
-						if (chunk->ptr != NULL) {
-							*(int8 *)ptr = res & 0xFF;
-							ptr++;
-						} else
-							*(int8 *)(base_ptr + chunk->offset) = res & 0xFF;
-							break;
+				/* Writing to the var: bits 8 to 15 have the VAR type */
+				if (chunk->ptr == NULL) ptr = base_ptr + chunk->offset;
 
-					case OC_VAR_U8:
-						/* Write the data */
-						if (chunk->ptr != NULL) {
-							*(uint8 *)ptr = res & 0xFF;
-							ptr++;
-						} else
-							*(uint8 *)(base_ptr + chunk->offset) = res & 0xFF;
-							break;
-
-					case OC_VAR_U16:
-						/* Write the data */
-						if (chunk->ptr != NULL) {
-							*(uint16 *)ptr = res & 0xFFFF;
-							ptr += 2;
-						} else
-							*(uint16 *)(base_ptr + chunk->offset) = res & 0xFFFF;
-							break;
-
-					case OC_VAR_I16:
-						/* Write the data */
-						if (chunk->ptr != NULL) {
-							*(int16 *)ptr = res & 0xFFFF;
-							ptr += 2;
-						} else
-							*(int16 *)(base_ptr + chunk->offset) = res & 0xFFFF;
-							break;
-
-					case OC_VAR_U32:
-						/* Write the data */
-						if (chunk->ptr != NULL) {
-							*(uint32 *)ptr = res;
-							ptr += 4;
-						} else
-							*(uint32 *)(base_ptr + chunk->offset) = res;
-							break;
-
-					case OC_VAR_I32:
-						/* Write the data */
-						if (chunk->ptr != NULL) {
-							*(int32 *)ptr = res;
-							ptr += 4;
-						} else
-							*(int32 *)(base_ptr + chunk->offset) = res;
-							break;
-
-					case OC_VAR_I64:
-						/* Write the data */
-						if (chunk->ptr != NULL) {
-							*(int64 *)ptr = res;
-							ptr += 8;
-						} else
-							*(int64 *)(base_ptr + chunk->offset) = res;
-							break;
+				/* Write the data */
+				switch (GetOldChunkVarType(chunk->type)) {
+					case OC_VAR_I8: *(int8  *)ptr = GB(res, 0, 8); break;
+					case OC_VAR_U8: *(uint8 *)ptr = GB(res, 0, 8); break;
+					case OC_VAR_I16:*(int16 *)ptr = GB(res, 0, 16); break;
+					case OC_VAR_U16:*(uint16*)ptr = GB(res, 0, 16); break;
+					case OC_VAR_I32:*(int32 *)ptr = res; break;
+					case OC_VAR_U32:*(uint32*)ptr = res; break;
+					case OC_VAR_I64:*(int64 *)ptr = res; break;
+					default: NOT_REACHED();
 				}
+
+				/* Increase pointer base for arrays when looping */
+				if (chunk->amount > 1 && chunk->ptr != NULL) ptr += CalcOldVarLen(chunk->type);
 			}
 		}
 
@@ -345,9 +295,6 @@ static void FixOldTowns(void)
 
 	/* Convert town-names if needed */
 	FOR_ALL_TOWNS(town) {
-		if (town->xy == 0)
-			continue;
-
 		if (IS_INT_INSIDE(town->townnametype, 0x20C1, 0x20C3)) {
 			town->townnametype = SPECSTR_TOWNNAME_ENGLISH + _opt.town_name;
 			town->townnameparts = GetOldTownName(town->townnameparts, _opt.town_name);
@@ -361,7 +308,7 @@ static void FixOldStations(void)
 
 	FOR_ALL_STATIONS(st) {
 		/* Check if we need to swap width and height for the station */
-		if (st->train_tile != 0 && _m[st->train_tile].m5 & 1) {
+		if (st->train_tile != 0 && GetRailStationAxis(st->train_tile) != AXIS_X) {
 			swap_byte(&st->trainst_w, &st->trainst_h);
 		}
 
@@ -374,7 +321,7 @@ static void FixOldStations(void)
 			st->bus_stops->station = st->index;
 			st->bus_stops->next = NULL;
 			st->bus_stops->prev = NULL;
-			st->bus_stops->slot[0] = st->bus_stops->slot[1] = INVALID_SLOT;
+			st->bus_stops->num_vehicles = 0;
 		}
 
 		if (st->lorry_tile_obsolete != 0) {
@@ -385,7 +332,7 @@ static void FixOldStations(void)
 			st->truck_stops->station = st->index;
 			st->truck_stops->next = NULL;
 			st->truck_stops->prev = NULL;
-			st->truck_stops->slot[0] = st->truck_stops->slot[1] = INVALID_SLOT;
+			st->truck_stops->num_vehicles = 0;
 		}
 	}
 }
@@ -393,26 +340,21 @@ static void FixOldStations(void)
 static void FixOldVehicles(void)
 {
 	/* Check for shared orders, and link them correctly */
-	{
-		Vehicle *v;
+	Vehicle* v;
 
-		FOR_ALL_VEHICLES(v) {
-			Vehicle *u;
+	FOR_ALL_VEHICLES(v) {
+		Vehicle *u;
 
-			if (v->type == 0)
-				continue;
+		/* We haven't used this bit for stations for ages */
+		if (v->type == VEH_Road) CLRBIT(v->u.road.state, 2);
 
-			FOR_ALL_VEHICLES_FROM(u, v->index + 1) {
-				if (u->type == 0)
-					continue;
-
-					/* If a vehicle has the same orders, add the link to eachother
-				in both vehicles */
-				if (v->orders == u->orders) {
-					v->next_shared = u;
-					u->prev_shared = v;
-					break;
-				}
+		FOR_ALL_VEHICLES_FROM(u, v->index + 1) {
+			/* If a vehicle has the same orders, add the link to eachother
+			 * in both vehicles */
+			if (v->orders == u->orders) {
+				v->next_shared = u;
+				u->prev_shared = v;
+				break;
 			}
 		}
 	}
@@ -432,7 +374,7 @@ static void FixOldVehicles(void)
  *  - OCL_CHUNK: load an other proc to load a part of the savegame, 'amount' times
  *  - OCL_ASSERT: to check if we are really at the place we expect to be.. because old savegames are too binary to be sure ;)
  */
-#define OCL_SVAR(type, base, offset)         { type,          1, NULL,    offsetof(base, offset), NULL }
+#define OCL_SVAR(type, base, offset)         { type,          1, NULL,    (uint)offsetof(base, offset), NULL }
 #define OCL_VAR(type, amount, pointer)       { type,     amount, pointer, 0,                      NULL }
 #define OCL_END()                                   { OC_END,        0, NULL,    0,                      NULL }
 #define OCL_NULL(amount)                            { OC_NULL,  amount, NULL,    0,                      NULL }
@@ -448,9 +390,8 @@ static void FixOldVehicles(void)
 
 extern TileIndex _animated_tile_list[256];
 extern char _name_array[512][32];
-extern uint16 _custom_sprites_base;
 
-static byte   _old_vehicle_multipler;
+static byte   _old_vehicle_multiplier;
 static uint8  _old_map3[OLD_MAP_SIZE * 2];
 static bool   _new_ttdpatch_format;
 static uint32 _old_town_index;
@@ -461,41 +402,34 @@ static void ReadTTDPatchFlags(void)
 {
 	int i;
 
-	if (_read_ttdpatch_flags)
-		return;
+	if (_read_ttdpatch_flags) return;
 
 	_read_ttdpatch_flags = true;
 
 	/* TTDPatch misuses _old_map3 for flags.. read them! */
-	_old_vehicle_multipler = _old_map3[0];
+	_old_vehicle_multiplier = _old_map3[0];
 	/* Somehow.... there was an error in some savegames, so 0 becomes 1
 	and 1 becomes 2. The rest of the values are okay */
-	if (_old_vehicle_multipler < 2)
-		_old_vehicle_multipler++;
+	if (_old_vehicle_multiplier < 2) _old_vehicle_multiplier++;
 
 	/* TTDPatch incraeses the Vehicle-part in the middle of the game,
 	so if the multipler is anything else but 1, the assert fails..
 	bump the assert value so it doesn't!
 	(1 multipler == 850 vehicles
 	1 vehicle   == 128 bytes */
-	_bump_assert_value = (_old_vehicle_multipler - 1) * 850 * 128;
-
-	_new_ttdpatch_format   = false;
+	_bump_assert_value = (_old_vehicle_multiplier - 1) * 850 * 128;
 
 	/* Check if we have a modern TTDPatch savegame (has extra data all around) */
-	if (memcmp(&_old_map3[0x1FFFA], "TTDp", 4) == 0)
-		_new_ttdpatch_format = true;
+	_new_ttdpatch_format = (memcmp(&_old_map3[0x1FFFA], "TTDp", 4) == 0);
 
 	/* Clean the misused places */
-	for (i = 0; i < 17; i++)
-		_old_map3[i] = 0;
-	for (i = 0x1FE00; i < 0x20000; i++)
-		_old_map3[i] = 0;
+	for (i = 0;       i < 17;      i++) _old_map3[i] = 0;
+	for (i = 0x1FE00; i < 0x20000; i++) _old_map3[i] = 0;
 
 	if (_new_ttdpatch_format)
 		DEBUG(oldloader, 1)("[OldLoader] Found TTDPatch game");
 
-	DEBUG(oldloader, 1)("[OldLoader] Vehicle-multipler is set to %d (%d vehicles)", _old_vehicle_multipler, _old_vehicle_multipler * 850);
+	DEBUG(oldloader, 1)("[OldLoader] Vehicle-multiplier is set to %d (%d vehicles)", _old_vehicle_multiplier, _old_vehicle_multiplier * 850);
 }
 
 static const OldChunks town_chunk[] = {
@@ -553,7 +487,7 @@ static const OldChunks town_chunk[] = {
 };
 static bool LoadOldTown(LoadgameState *ls, int num)
 {
-	if (!AddBlockIfNeeded(&_town_pool, num))
+	if (!AddBlockIfNeeded(&_Town_pool, num))
 		error("Towns: failed loading savegame: too many towns");
 
 	return LoadChunk(ls, GetTown(num), town_chunk);
@@ -564,13 +498,13 @@ static const OldChunks order_chunk[] = {
 	OCL_VAR ( OC_UINT16,   1, &_old_order ),
 	OCL_END()
 };
+
 static bool LoadOldOrder(LoadgameState *ls, int num)
 {
-	if (!AddBlockIfNeeded(&_order_pool, num))
+	if (!AddBlockIfNeeded(&_Order_pool, num))
 		error("Orders: failed loading savegame: too many orders");
 
-	if (!LoadChunk(ls, NULL, order_chunk))
-		return false;
+	if (!LoadChunk(ls, NULL, order_chunk)) return false;
 
 	AssignOrder(GetOrder(num), UnpackOldOrder(_old_order));
 
@@ -588,15 +522,15 @@ static const OldChunks depot_chunk[] = {
 	OCL_VAR ( OC_UINT32,   1, &_old_town_index ),
 	OCL_END()
 };
+
 static bool LoadOldDepot(LoadgameState *ls, int num)
 {
-	if (!AddBlockIfNeeded(&_depot_pool, num))
+	if (!AddBlockIfNeeded(&_Depot_pool, num))
 		error("Depots: failed loading savegame: too many depots");
 
-	if (!LoadChunk(ls, GetDepot(num), depot_chunk))
-		return false;
+	if (!LoadChunk(ls, GetDepot(num), depot_chunk)) return false;
 
-	if (GetDepot(num)->xy != 0) {
+	if (IsValidDepot(GetDepot(num))) {
 		GetDepot(num)->town_index = REMAP_TOWN_IDX(_old_town_index);
 	}
 
@@ -610,10 +544,10 @@ static const OldChunks price_chunk[] = {
 	OCL_VAR ( OC_UINT16,   1, &_old_price_frac ),
 	OCL_END()
 };
+
 static bool LoadOldPrice(LoadgameState *ls, int num)
 {
-	if (!LoadChunk(ls, NULL, price_chunk))
-		return false;
+	if (!LoadChunk(ls, NULL, price_chunk)) return false;
 
 	/* We use a struct to store the prices, but they are ints in a row..
 	so just access the struct as an array of int32's */
@@ -630,10 +564,10 @@ static const OldChunks cargo_payment_rate_chunk[] = {
 	OCL_NULL( 2 ),         // Junk
 	OCL_END()
 };
+
 static bool LoadOldCargoPaymentRate(LoadgameState *ls, int num)
 {
-	if (!LoadChunk(ls, NULL, cargo_payment_rate_chunk))
-		return false;
+	if (!LoadChunk(ls, NULL, cargo_payment_rate_chunk)) return false;
 
 	_cargo_payment_rates[num] = -_old_price;
 	_cargo_payment_rates_frac[num] = _old_price_frac;
@@ -655,6 +589,7 @@ static const OldChunks goods_chunk[] = {
 
 	OCL_END()
 };
+
 static bool LoadOldGood(LoadgameState *ls, int num)
 {
 	Station *st = GetStation(_current_station_id);
@@ -694,7 +629,7 @@ static const OldChunks station_chunk[] = {
 	OCL_SVAR(  OC_UINT8, Station, blocked_months_obsolete ),
 	OCL_NULL( 1 ),         // Unknown
 	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Station, airport_flags ),
-	OCL_SVAR( OC_UINT16, Station, last_vehicle ),
+	OCL_NULL( 2 ),         // last_vehicle. now last_vehicle_type
 
 	OCL_NULL( 4 ),         // Junk at end of chunk
 
@@ -704,7 +639,7 @@ static bool LoadOldStation(LoadgameState *ls, int num)
 {
 	Station *st;
 
-	if (!AddBlockIfNeeded(&_station_pool, num))
+	if (!AddBlockIfNeeded(&_Station_pool, num))
 		error("Stations: failed loading savegame: too many stations");
 
 	st = GetStation(num);
@@ -713,7 +648,7 @@ static bool LoadOldStation(LoadgameState *ls, int num)
 	if (!LoadChunk(ls, st, station_chunk))
 		return false;
 
-	if (st->xy != 0) {
+	if (IsValidStation(st)) {
 		if (st->train_tile) {
 			/* Calculate the trainst_w and trainst_h */
 			uint w = GB(_old_platforms, 3, 3);
@@ -764,8 +699,8 @@ static const OldChunks industry_chunk[] = {
 
 	OCL_SVAR(  OC_UINT8, Industry, type ),
 	OCL_SVAR(  OC_UINT8, Industry, owner ),
-	OCL_SVAR(  OC_UINT8, Industry, color_map ),
-	OCL_SVAR(  OC_UINT8, Industry, last_prod_year ),
+	OCL_SVAR(  OC_UINT8, Industry, random_color ),
+	OCL_SVAR( OC_FILE_U8 | OC_VAR_I32, Industry, last_prod_year ),
 	OCL_SVAR( OC_UINT16, Industry, counter ),
 	OCL_SVAR(  OC_UINT8, Industry, was_cargo_delivered ),
 
@@ -773,18 +708,18 @@ static const OldChunks industry_chunk[] = {
 
 	OCL_END()
 };
+
 static bool LoadOldIndustry(LoadgameState *ls, int num)
 {
 	Industry *i;
 
-	if (!AddBlockIfNeeded(&_industry_pool, num))
+	if (!AddBlockIfNeeded(&_Industry_pool, num))
 		error("Industries: failed loading savegame: too many industries");
 
 	i = GetIndustry(num);
-	if (!LoadChunk(ls, i, industry_chunk))
-		return false;
+	if (!LoadChunk(ls, i, industry_chunk)) return false;
 
-	if (i->xy != 0) {
+	if (IsValidIndustry(i)) {
 		i->town = GetTown(REMAP_TOWN_IDX(_old_town_index));
 	}
 
@@ -792,21 +727,20 @@ static bool LoadOldIndustry(LoadgameState *ls, int num)
 }
 
 static PlayerID _current_player_id;
-static uint16 _old_inaugurated_year;
 static int32 _old_yearly;
 
 static const OldChunks player_yearly_chunk[] = {
 	OCL_VAR(  OC_INT32,   1, &_old_yearly ),
 	OCL_END()
 };
+
 static bool OldPlayerYearly(LoadgameState *ls, int num)
 {
 	int i;
 	Player *p = GetPlayer(_current_player_id);
 
 	for (i = 0; i < 13; i++) {
-		if (!LoadChunk(ls, NULL, player_yearly_chunk))
-			return false;
+		if (!LoadChunk(ls, NULL, player_yearly_chunk)) return false;
 
 		p->yearly_expenses[num][i] = _old_yearly;
 	}
@@ -823,21 +757,20 @@ static const OldChunks player_economy_chunk[] = {
 
 	OCL_END()
 };
+
 static bool OldPlayerEconomy(LoadgameState *ls, int num)
 {
 	int i;
 	Player *p = GetPlayer(_current_player_id);
 
-	if (!LoadChunk(ls, &p->cur_economy, player_economy_chunk))
-		return false;
+	if (!LoadChunk(ls, &p->cur_economy, player_economy_chunk)) return false;
 
 	/* Don't ask, but the number in TTD(Patch) are inversed to OpenTTD */
 	p->cur_economy.income   = -p->cur_economy.income;
 	p->cur_economy.expenses = -p->cur_economy.expenses;
 
 	for (i = 0; i < 24; i++) {
-		if (!LoadChunk(ls, &p->old_economy[i], player_economy_chunk))
-			return false;
+		if (!LoadChunk(ls, &p->old_economy[i], player_economy_chunk)) return false;
 
 		p->old_economy[i].income   = -p->old_economy[i].income;
 		p->old_economy[i].expenses = -p->old_economy[i].expenses;
@@ -862,6 +795,7 @@ static const OldChunks player_ai_build_rec_chunk[] = {
 
 	OCL_END()
 };
+
 static bool OldLoadAIBuildRec(LoadgameState *ls, int num)
 {
 	Player *p = GetPlayer(_current_player_id);
@@ -977,6 +911,7 @@ static const OldChunks player_ai_chunk[] = {
 
 	OCL_END()
 };
+
 static bool OldPlayerAI(LoadgameState *ls, int num)
 {
 	Player *p = GetPlayer(_current_player_id);
@@ -1006,9 +941,9 @@ static const OldChunks player_chunk[] = {
 	OCL_CHUNK( 3, OldPlayerYearly ),
 	OCL_CHUNK( 1, OldPlayerEconomy ),
 
-	OCL_VAR ( OC_UINT16,   1,    &_old_inaugurated_year ),
-	OCL_SVAR(   OC_TILE, Player, last_build_coordinate ),
-	OCL_SVAR(  OC_UINT8, Player, num_valid_stat_ent ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_I32, Player, inaugurated_year),
+	OCL_SVAR(                  OC_TILE, Player, last_build_coordinate ),
+	OCL_SVAR(                 OC_UINT8, Player, num_valid_stat_ent ),
 
 	OCL_CHUNK( 1, OldPlayerAI ),
 
@@ -1025,14 +960,14 @@ static const OldChunks player_chunk[] = {
 
 	OCL_END()
 };
+
 static bool LoadOldPlayer(LoadgameState *ls, int num)
 {
 	Player *p = GetPlayer(num);
 
 	_current_player_id = num;
 
-	if (!LoadChunk(ls, p, player_chunk))
-		return false;
+	if (!LoadChunk(ls, p, player_chunk)) return false;
 
 	p->name_1 = RemapOldStringID(_old_string_id);
 	p->president_name_1 = RemapOldStringID(_old_string_id_2);
@@ -1045,7 +980,7 @@ static bool LoadOldPlayer(LoadgameState *ls, int num)
 	} else {
 		/* Beside some multiplayer maps (1 on 1), which we don't official support,
 		all other players are an AI.. mark them as such */
-		p->is_ai = 1;
+		p->is_ai = true;
 	}
 
 	/* Sometimes it is better to not ask.. in old scenarios, the money
@@ -1057,7 +992,7 @@ static bool LoadOldPlayer(LoadgameState *ls, int num)
 		p->money64 = p->player_money = p->current_loan = 100000;
 
 	_player_colors[num] = p->player_color;
-	p->inaugurated_year = _old_inaugurated_year - MAX_YEAR_BEGIN_REAL;
+	p->inaugurated_year -= ORIGINAL_BASE_YEAR;
 	if (p->location_of_house == 0xFFFF)
 		p->location_of_house = 0;
 
@@ -1065,7 +1000,7 @@ static bool LoadOldPlayer(LoadgameState *ls, int num)
 	 * really figured out as of now, p->ai.cur_veh; needed for 'sell vehicle'
 	 * is NULL and the function will crash. To fix this, just change the state
 	 * to some harmless state, like 'loop vehicle'; 1 */
-	if (!IS_HUMAN_PLAYER(num) && p->ai.state == 20) p->ai.state = 1;
+	if (!IsHumanPlayer(num) && p->ai.state == 20) p->ai.state = 1;
 
 	if (p->is_ai && (!_networking || _network_server) && _ai.enabled)
 		AI_StartNewAI(p->index);
@@ -1087,10 +1022,11 @@ static const OldChunks vehicle_train_chunk[] = {
 
 	OCL_END()
 };
+
 static const OldChunks vehicle_road_chunk[] = {
 	OCL_SVAR(  OC_UINT8, VehicleRoad, state ),
 	OCL_SVAR(  OC_UINT8, VehicleRoad, frame ),
-	OCL_SVAR( OC_UINT16, VehicleRoad, unk2 ),
+	OCL_SVAR( OC_UINT16, VehicleRoad, blocked_ctr ),
 	OCL_SVAR(  OC_UINT8, VehicleRoad, overtaking ),
 	OCL_SVAR(  OC_UINT8, VehicleRoad, overtaking_ctr ),
 	OCL_SVAR( OC_UINT16, VehicleRoad, crashed_ctr ),
@@ -1100,6 +1036,7 @@ static const OldChunks vehicle_road_chunk[] = {
 
 	OCL_END()
 };
+
 static const OldChunks vehicle_ship_chunk[] = {
 	OCL_SVAR(  OC_UINT8, VehicleShip, state ),
 
@@ -1107,6 +1044,7 @@ static const OldChunks vehicle_ship_chunk[] = {
 
 	OCL_END()
 };
+
 static const OldChunks vehicle_air_chunk[] = {
 	OCL_SVAR(  OC_UINT8, VehicleAir, pos ),
 	OCL_SVAR(  OC_FILE_U8 | OC_VAR_U16, VehicleAir, targetairport ),
@@ -1117,6 +1055,7 @@ static const OldChunks vehicle_air_chunk[] = {
 
 	OCL_END()
 };
+
 static const OldChunks vehicle_special_chunk[] = {
 	OCL_SVAR( OC_UINT16, VehicleSpecial, unk0 ),
 	OCL_SVAR(  OC_UINT8, VehicleSpecial, unk2 ),
@@ -1125,6 +1064,7 @@ static const OldChunks vehicle_special_chunk[] = {
 
 	OCL_END()
 };
+
 static const OldChunks vehicle_disaster_chunk[] = {
 	OCL_SVAR( OC_UINT16, VehicleDisaster, image_override ),
 	OCL_SVAR( OC_UINT16, VehicleDisaster, unk2 ),
@@ -1133,11 +1073,13 @@ static const OldChunks vehicle_disaster_chunk[] = {
 
 	OCL_END()
 };
+
 static const OldChunks vehicle_empty_chunk[] = {
 	OCL_NULL( 10 ), // Junk
 
 	OCL_END()
 };
+
 static bool LoadOldVehicleUnion(LoadgameState *ls, int num)
 {
 	Vehicle *v = GetVehicle(_current_vehicle_id);
@@ -1177,7 +1119,7 @@ static const OldChunks vehicle_chunk[] = {
 	OCL_SVAR(  OC_UINT8, Vehicle, cur_order_index ),
 	OCL_SVAR(   OC_TILE, Vehicle, dest_tile ),
 	OCL_SVAR( OC_UINT16, Vehicle, load_unload_time_rem ),
-	OCL_SVAR( OC_UINT16, Vehicle, date_of_last_service ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Vehicle, date_of_last_service ),
 	OCL_SVAR( OC_UINT16, Vehicle, service_interval ),
 	OCL_SVAR( OC_FILE_U8 | OC_VAR_U16, Vehicle, last_station_visited ),
 	OCL_SVAR(  OC_UINT8, Vehicle, tick_counter ),
@@ -1211,9 +1153,9 @@ static const OldChunks vehicle_chunk[] = {
 	OCL_SVAR( OC_FILE_U8 | OC_VAR_U16, Vehicle, cargo_source ),
 	OCL_SVAR(  OC_UINT8, Vehicle, cargo_days ),
 
-	OCL_SVAR( OC_UINT16, Vehicle, age ),
-	OCL_SVAR( OC_UINT16, Vehicle, max_age ),
-	OCL_SVAR(  OC_UINT8, Vehicle, build_year ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Vehicle, age ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Vehicle, max_age ),
+	OCL_SVAR( OC_FILE_U8 | OC_VAR_I32, Vehicle, build_year ),
 	OCL_SVAR( OC_FILE_U8 | OC_VAR_U16, Vehicle, unitnumber ),
 
 	OCL_SVAR( OC_UINT16, Vehicle, engine_type ),
@@ -1244,6 +1186,7 @@ static const OldChunks vehicle_chunk[] = {
 
 	OCL_END()
 };
+
 static bool LoadOldVehicle(LoadgameState *ls, int num)
 {
 	uint i;
@@ -1251,17 +1194,16 @@ static bool LoadOldVehicle(LoadgameState *ls, int num)
 	/* Read the TTDPatch flags, because we need some info from it */
 	ReadTTDPatchFlags();
 
-	for (i = 0; i < _old_vehicle_multipler; i++) {
+	for (i = 0; i < _old_vehicle_multiplier; i++) {
 		Vehicle *v;
 
-		_current_vehicle_id = num * _old_vehicle_multipler + i;
+		_current_vehicle_id = num * _old_vehicle_multiplier + i;
 
-		if (!AddBlockIfNeeded(&_vehicle_pool, _current_vehicle_id))
+		if (!AddBlockIfNeeded(&_Vehicle_pool, _current_vehicle_id))
 			error("Vehicles: failed loading savegame: too many vehicles");
 
 		v = GetVehicle(_current_vehicle_id);
-		if (!LoadChunk(ls, v, vehicle_chunk))
-			return false;
+		if (!LoadChunk(ls, v, vehicle_chunk)) return false;
 
 		/* This should be consistent, else we have a big problem... */
 		if (v->index != _current_vehicle_id) {
@@ -1273,9 +1215,6 @@ static bool LoadOldVehicle(LoadgameState *ls, int num)
 			v->orders = GetOrder(REMAP_ORDER_IDX(_old_order_ptr));
 		}
 		AssignOrder(&v->current_order, UnpackOldOrder(_old_order));
-		/* TTDPatch maps sprites from 0x2000 up. */
-		if (v->cur_image >= 0x2000)
-			v->cur_image -= 0x2000 - _custom_sprites_base;
 
 		/* For some reason we need to correct for this */
 		switch (v->spritenum) {
@@ -1290,26 +1229,26 @@ static bool LoadOldVehicle(LoadgameState *ls, int num)
 		v->string_id = RemapOldStringID(_old_string_id);
 
 		/* Vehicle-subtype is different in TTD(Patch) */
-		if (v->type == VEH_Special)
-			v->subtype = v->subtype >> 1;
+		if (v->type == VEH_Special) v->subtype = v->subtype >> 1;
 	}
 
 	return true;
 }
 
 static const OldChunks sign_chunk[] = {
-	OCL_SVAR( OC_UINT16, SignStruct, str ),
-	OCL_SVAR( OC_FILE_U16 | OC_VAR_I32,SignStruct, x ),
-	OCL_SVAR( OC_FILE_U16 | OC_VAR_I32,SignStruct, y ),
-	OCL_SVAR( OC_FILE_U16 | OC_VAR_I8, SignStruct, z ),
+	OCL_SVAR( OC_UINT16, Sign, str ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_I32,Sign, x ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_I32,Sign, y ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_I8, Sign, z ),
 
 	OCL_NULL( 6 ),         // Width of sign, no longer in use
 
 	OCL_END()
 };
+
 static bool LoadOldSign(LoadgameState *ls, int num)
 {
-	if (!AddBlockIfNeeded(&_sign_pool, num))
+	if (!AddBlockIfNeeded(&_Sign_pool, num))
 		error("Signs: failed loading savegame: too many signs");
 
 	return LoadChunk(ls, GetSign(num), sign_chunk);
@@ -1317,8 +1256,8 @@ static bool LoadOldSign(LoadgameState *ls, int num)
 
 static const OldChunks engine_chunk[] = {
 	OCL_SVAR( OC_UINT16, Engine, player_avail ),
-	OCL_SVAR( OC_UINT16, Engine, intro_date ),
-	OCL_SVAR( OC_UINT16, Engine, age ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Engine, intro_date ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Engine, age ),
 	OCL_SVAR( OC_UINT16, Engine, reliability ),
 	OCL_SVAR( OC_UINT16, Engine, reliability_spd_dec ),
 	OCL_SVAR( OC_UINT16, Engine, reliability_start ),
@@ -1338,10 +1277,10 @@ static const OldChunks engine_chunk[] = {
 
 	OCL_END()
 };
+
 static bool LoadOldEngine(LoadgameState *ls, int num)
 {
-	if (!LoadChunk(ls, GetEngine(num), engine_chunk))
-		return false;
+	if (!LoadChunk(ls, GetEngine(num), engine_chunk)) return false;
 
 	/* Make sure wagons are marked as do-not-age */
 	if ((num >= 27 && num < 54) || (num >= 57 && num < 84) || (num >= 89 && num < 116))
@@ -1358,7 +1297,8 @@ static const OldChunks subsidy_chunk[] = {
 
 	OCL_END()
 };
-static bool LoadOldSubsidy(LoadgameState *ls, int num)
+
+static inline bool LoadOldSubsidy(LoadgameState *ls, int num)
 {
 	return LoadChunk(ls, &_subsidies[num], subsidy_chunk);
 }
@@ -1383,7 +1323,8 @@ static const OldChunks game_difficulty_chunk[] = {
 	OCL_SVAR( OC_FILE_U16 | OC_VAR_I32, GameDifficulty, disasters ),
 	OCL_END()
 };
-static bool LoadOldGameDifficulty(LoadgameState *ls, int num)
+
+static inline bool LoadOldGameDifficulty(LoadgameState *ls, int num)
 {
 	return LoadChunk(ls, &_opt.diff, game_difficulty_chunk);
 }
@@ -1413,6 +1354,7 @@ static bool LoadOldMapPart1(LoadgameState *ls, int num)
 
 	return !ls->failed;
 }
+
 static bool LoadOldMapPart2(LoadgameState *ls, int num)
 {
 	uint i;
@@ -1427,11 +1369,10 @@ static bool LoadOldMapPart2(LoadgameState *ls, int num)
 	return !ls->failed;
 }
 
-
 static uint32 _old_cur_town_ctr;
 static const OldChunks main_chunk[] = {
 	OCL_ASSERT( 0 ),
-	OCL_VAR ( OC_UINT16,   1, &_date ),
+	OCL_VAR ( OC_FILE_U16 | OC_VAR_U32, 1, &_date ),
 	OCL_VAR ( OC_UINT16,   1, &_date_fract ),
 	OCL_NULL( 600 ),            // TextEffects
 	OCL_VAR ( OC_UINT32,   2, &_random_seeds[0] ),
@@ -1507,7 +1448,7 @@ static const OldChunks main_chunk[] = {
 	OCL_VAR ( OC_FILE_U8 | OC_VAR_U16,    1, &_station_tick_ctr ),
 
 	OCL_VAR (  OC_UINT8,    1, &_opt.currency ),
-	OCL_VAR (  OC_UINT8,    1, &_opt.kilometers ),
+	OCL_VAR (  OC_UINT8,    1, &_opt.units ),
 	OCL_VAR ( OC_FILE_U8 | OC_VAR_U32,    1, &_cur_player_tick_index ),
 
 	OCL_NULL( 2 ),               // Date stuff, calculated automatically
@@ -1544,6 +1485,7 @@ static const OldChunks main_chunk[] = {
 
 	OCL_END()
 };
+
 static bool LoadOldMain(LoadgameState *ls)
 {
 	int i;
@@ -1572,16 +1514,30 @@ static bool LoadOldMain(LoadgameState *ls)
 	}
 
 	for (i = 0; i < OLD_MAP_SIZE; i ++) {
-		if (IsTileType(i, MP_RAILWAY)) {
-			/* We save presignals different from TTDPatch, convert them */
-			if (GB(_m[i].m5, 6, 2) == 1) {
-				/* This byte is always zero in TTD for this type of tile */
-				if (_m[i].m4) /* Convert the presignals to our own format */
-					_m[i].m4 = (_m[i].m4 >> 1) & 7;
-			}
-			/* TTDPatch stores PBS things in L6 and all elsewhere; so we'll just
-			* clear it for ourselves and let OTTD's rebuild PBS itself */
-			_m[i].m4 &= 0xF; /* Only keep the lower four bits; upper four is PBS */
+		switch (GetTileType(i)) {
+			case MP_STATION: _m[i].m4 = 0; break; // We don't understand this grf mapping (yet)
+			case MP_RAILWAY:
+				/* We save presignals different from TTDPatch, convert them */
+				if (GetRailTileType(i) == RAIL_TILE_SIGNALS) {
+					/* This byte is always zero in TTD for this type of tile */
+					if (_m[i].m4) /* Convert the presignals to our own format */
+						_m[i].m4 = (_m[i].m4 >> 1) & 7;
+				}
+				/* TTDPatch stores PBS things in L6 and all elsewhere; so we'll just
+				 * clear it for ourselves and let OTTD's rebuild PBS itself */
+				_m[i].m4 &= 0xF; /* Only keep the lower four bits; upper four is PBS */
+				break;
+			case MP_WATER: {
+				/* TTDPatch has all tiles touching water as coast (water)-type, we don't.
+				 * This is only true from a certain TTDP version, but there is no harm
+				 * in checking all the time */
+				Slope s = GetTileSlope(i, NULL);
+				if (s == SLOPE_ENW || s == SLOPE_NWS || s == SLOPE_SEN || s == SLOPE_WSE || IsSteepSlope(s)) {
+					SetTileType(i, MP_CLEAR);
+					SetTileOwner(i, OWNER_NONE);
+				}
+			default: break;
+			} break;
 		}
 	}
 
@@ -1627,24 +1583,19 @@ bool LoadOldSaveGame(const char *file)
 	return true;
 }
 
-void GetOldSaveGameName(char *title, const char *file)
+void GetOldSaveGameName(char *title, const char *path, const char *file)
 {
+	char filename[MAX_PATH];
 	FILE *f;
 
-	f = fopen(file, "rb");
-	title[0] = 0;
-	title[48] = 0;
+	snprintf(filename, lengthof(filename), "%s" PATHSEP "%s", path, file);
+	f = fopen(filename, "rb");
+	title[0] = '\0';
+	title[48] = '\0';
 
-	if (f == NULL)
-		return;
+	if (f == NULL) return;
 
-	if (fread(title, 1, 48, f) != 48)
-		snprintf(title, 48, "Corrupt file");
+	if (fread(title, 1, 48, f) != 48) snprintf(title, 48, "Corrupt file");
 
 	fclose(f);
 }
-
-void GetOldScenarioGameName(char *title, const char *file)
-{
-	GetOldSaveGameName(title, file);
-}
