@@ -1,4 +1,4 @@
-/* $Id: openttd.c 9853 2007-05-15 22:02:32Z rubidium $ */
+/* $Id: openttd.c 10841 2007-08-10 17:53:12Z rubidium $ */
 
 #include "stdafx.h"
 #include "string.h"
@@ -25,6 +25,7 @@
 #include "station_map.h"
 #include "town_map.h"
 #include "tunnel_map.h"
+#include "water_map.h"
 #include "vehicle.h"
 #include "viewport.h"
 #include "window.h"
@@ -419,6 +420,11 @@ int ttd_main(int argc, char *argv[])
 	if (startyear != INVALID_YEAR) _patches_newgame.starting_year = startyear;
 	if (generation_seed != GENERATE_NEW_SEED) _patches_newgame.generation_seed = generation_seed;
 
+	/* The width and height must be at least 1 pixel, this
+	 * way all internal drawing routines work correctly. */
+	if (_cur_resolution[0] == 0) _cur_resolution[0] = 1;
+	if (_cur_resolution[1] == 0) _cur_resolution[1] = 1;
+
 	if (_dedicated_forks && !dedicated) _dedicated_forks = false;
 
 	// enumerate language files
@@ -1587,13 +1593,42 @@ bool AfterLoadGame(void)
 		SettingsDisableElrail(_patches.disable_elrails);
 	}
 
+	if (CheckSavegameVersion(41)) {
+		Vehicle *v;
+		FOR_ALL_VEHICLES(v) {
+			if (IsBridgeTile(v->tile) && IsBridgeRamp(v->tile) &&
+					(v->z_pos & (TILE_HEIGHT - 1)) == (TILE_HEIGHT - 1)) {
+				/* Under some circumstances the trains going up a ramp can be one
+				* pixel too low when they enter the bridge. This causes the train
+				* to "disappear" under the bridge, which causes the train to
+				* break into two pieces and crash slightly later.
+				*
+				* This "hack" will make the trains on those positions on the
+				* ramps run one pixel higher. This offset will be automatically
+				* "fixed" when it enters the bridge middle parts or when it
+				* drives down the ramp of the bridge.
+				*/
+				v->z_pos++;
+			}
+		}
+	}
+
 	/* Buoys do now store the owner of the previous water tile, which can never
 	 * be OWNER_NONE. So replace OWNER_NONE with OWNER_WATER. */
 	if (CheckSavegameVersion(46)) {
+		TileIndex t;
 		Station *st;
 		FOR_ALL_STATIONS(st) {
 			if (IsBuoy(st) && IsTileOwner(st->xy, OWNER_NONE)) SetTileOwner(st->xy, OWNER_WATER);
 		}
+
+		/* Locks/shiplifts in very old savegames had OWNER_WATER as owner */
+		for (t = 0; t < MapSize(); t++) {
+			if (IsTileType(t, MP_WATER) && GetWaterTileType(t) == WATER_LOCK &&
+					GetTileOwner(t) == OWNER_WATER) {
+				SetTileOwner(t, OWNER_NONE);
+			}
+		}
 	}
 
 	if (CheckSavegameVersion(7)) {
@@ -1611,6 +1646,13 @@ bool AfterLoadGame(void)
 		}
 	}
 
+	{
+		Vehicle *v;
+		FOR_ALL_VEHICLES(v) {
+			if (v->type == VEH_Road && v->u.road.state >= 250) SETBIT(v->u.road.state, 2);
+		}
+	}
+
 	return true;
 }
 
