@@ -1,4 +1,4 @@
-/* $Id: player_gui.c 7415 2006-12-07 13:00:41Z Darkvater $ */
+/* $Id: player_gui.c 11074 2007-09-09 20:52:54Z rubidium $ */
 
 #include "stdafx.h"
 #include "openttd.h"
@@ -825,7 +825,7 @@ static void PlayerCompanyWndProc(Window *w, WindowEvent *e)
 			break;
 
 		case WE_PLACE_OBJ:
-			if (DoCommandP(e->we.place.tile, 0, 0, NULL, CMD_BUILD_COMPANY_HQ | CMD_AUTO | CMD_NO_WATER | CMD_MSG(STR_7071_CAN_T_BUILD_COMPANY_HEADQUARTERS)))
+			if (DoCommandP(e->we.place.tile, 0, 0, NULL, CMD_BUILD_COMPANY_HQ | CMD_NO_WATER | CMD_MSG(STR_7071_CAN_T_BUILD_COMPANY_HEADQUARTERS)))
 				ResetObjectToPlace();
 				w->widget[PCW_WIDGET_BUILD_VIEW_HQ].type = WWT_PUSHTXTBTN; // this button can now behave as a normal push button
 				RaiseWindowButtons(w);
